# Copyright (c) 2014 VMware, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient import api_versions
from novaclient import exceptions
from novaclient.tests.unit.fixture_data import client
from novaclient.tests.unit.fixture_data import server_groups as data
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import server_groups


class ServerGroupsTest(utils.FixturedTestCase):

    client_fixture_class = client.V1
    data_fixture_class = data.Fixture

    def test_list_server_groups(self):
        result = self.cs.server_groups.list()
        self.assert_request_id(result, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/os-server-groups')
        self.assertEqual(4, len(result))
        for server_group in result:
            self.assertIsInstance(server_group,
                                  server_groups.ServerGroup)

    def test_list_server_groups_with_all_projects(self):
        result = self.cs.server_groups.list(all_projects=True)
        self.assert_request_id(result, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/os-server-groups?all_projects=True')
        self.assertEqual(8, len(result))
        for server_group in result:
            self.assertIsInstance(server_group,
                                  server_groups.ServerGroup)

    def test_list_server_groups_with_limit_and_offset(self):
        all_groups = self.cs.server_groups.list()
        result = self.cs.server_groups.list(limit=2, offset=1)
        self.assert_request_id(result, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/os-server-groups?limit=2&offset=1')
        self.assertEqual(2, len(result))
        for server_group in result:
            self.assertIsInstance(server_group,
                                  server_groups.ServerGroup)
        self.assertEqual(all_groups[1:3], result)

    def test_create_server_group(self):
        kwargs = {'name': 'ig1',
                  'policies': ['anti-affinity']}
        server_group = self.cs.server_groups.create(**kwargs)
        self.assert_request_id(server_group, fakes.FAKE_REQUEST_ID_LIST)
        body = {'server_group': kwargs}
        self.assert_called('POST', '/os-server-groups', body)
        self.assertIsInstance(server_group,
                              server_groups.ServerGroup)

    def test_get_server_group(self):
        id = '2cbd51f4-fafe-4cdb-801b-cf913a6f288b'
        server_group = self.cs.server_groups.get(id)
        self.assert_request_id(server_group, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/os-server-groups/%s' % id)
        self.assertIsInstance(server_group,
                              server_groups.ServerGroup)

    def test_delete_server_group(self):
        id = '2cbd51f4-fafe-4cdb-801b-cf913a6f288b'
        ret = self.cs.server_groups.delete(id)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/os-server-groups/%s' % id)

    def test_delete_server_group_object(self):
        id = '2cbd51f4-fafe-4cdb-801b-cf913a6f288b'
        server_group = self.cs.server_groups.get(id)
        ret = server_group.delete()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/os-server-groups/%s' % id)

    def test_find_server_groups_by_name(self):
        expected_name = 'ig1'
        kwargs = {self.cs.server_groups.resource_class.NAME_ATTR:
                  expected_name}
        server_group = self.cs.server_groups.find(**kwargs)
        self.assert_request_id(server_group, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/os-server-groups')
        self.assertIsInstance(server_group, server_groups.ServerGroup)
        actual_name = getattr(server_group,
                              self.cs.server_groups.resource_class.NAME_ATTR)
        self.assertEqual(expected_name, actual_name)

    def test_find_no_existing_server_groups_by_name(self):
        expected_name = 'no-exist'
        kwargs = {self.cs.server_groups.resource_class.NAME_ATTR:
                  expected_name}
        self.assertRaises(exceptions.NotFound,
                          self.cs.server_groups.find,
                          **kwargs)
        self.assert_called('GET', '/os-server-groups')


class ServerGroupsTestV264(ServerGroupsTest):
    def setUp(self):
        super(ServerGroupsTestV264, self).setUp()
        self.cs.api_version = api_versions.APIVersion("2.64")

    def test_create_server_group(self):
        name = 'ig1'
        policy = 'anti-affinity'
        server_group = self.cs.server_groups.create(name, policy)
        self.assert_request_id(server_group, fakes.FAKE_REQUEST_ID_LIST)
        body = {'server_group': {'name': name, 'policy': policy}}
        self.assert_called('POST', '/os-server-groups', body)
        self.assertIsInstance(server_group,
                              server_groups.ServerGroup)

    def test_create_server_group_with_rules(self):
        kwargs = {'name': 'ig1',
                  'policy': 'anti-affinity',
                  'rules': {'max_server_per_host': 3}}
        server_group = self.cs.server_groups.create(**kwargs)
        self.assert_request_id(server_group, fakes.FAKE_REQUEST_ID_LIST)
        body = {
            'server_group': {
                'name': 'ig1',
                'policy': 'anti-affinity',
                'rules': {'max_server_per_host': 3}
            }
        }
        self.assert_called('POST', '/os-server-groups', body)
        self.assertIsInstance(server_group,
                              server_groups.ServerGroup)



# Copyright 2013 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse

from novaclient import base


class InstanceUsageAuditLog(base.Resource):
    pass


class InstanceUsageAuditLogManager(base.Manager):
    resource_class = InstanceUsageAuditLog

    def get(self, before=None):
        """Get server usage audits.

        :param before: Filters the response by the date and time
                       before which to list usage audits.
        """
        if before:
            return self._get('/os-instance_usage_audit_log/%s' %
                             parse.quote(before, safe=''),
                             'instance_usage_audit_log')
        else:
            return self._get('/os-instance_usage_audit_log',
                             'instance_usage_audit_logs')



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient import api_versions
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import flavor_access


class FlavorAccessTest(utils.TestCase):
    def setUp(self):
        super(FlavorAccessTest, self).setUp()
        self.cs = fakes.FakeClient(api_versions.APIVersion("2.0"))

    def test_list_access_by_flavor_private(self):
        kwargs = {'flavor': self.cs.flavors.get(2)}
        r = self.cs.flavor_access.list(**kwargs)
        self.assert_request_id(r, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/2/os-flavor-access')
        for a in r:
            self.assertIsInstance(a, flavor_access.FlavorAccess)

    def test_add_tenant_access(self):
        flavor = self.cs.flavors.get(2)
        tenant = 'proj2'
        r = self.cs.flavor_access.add_tenant_access(flavor, tenant)
        self.assert_request_id(r, fakes.FAKE_REQUEST_ID_LIST)

        body = {
            "addTenantAccess": {
                "tenant": "proj2"
            }
        }

        self.cs.assert_called('POST', '/flavors/2/action', body)
        for a in r:
            self.assertIsInstance(a, flavor_access.FlavorAccess)

    def test_remove_tenant_access(self):
        flavor = self.cs.flavors.get(2)
        tenant = 'proj2'
        r = self.cs.flavor_access.remove_tenant_access(flavor, tenant)
        self.assert_request_id(r, fakes.FAKE_REQUEST_ID_LIST)

        body = {
            "removeTenantAccess": {
                "tenant": "proj2"
            }
        }

        self.cs.assert_called('POST', '/flavors/2/action', body)
        for a in r:
            self.assertIsInstance(a, flavor_access.FlavorAccess)

    def test_repr_flavor_access(self):
        flavor = self.cs.flavors.get(2)
        tenant = 'proj3'
        r = self.cs.flavor_access.add_tenant_access(flavor, tenant)

        def get_expected(flavor_access):
            return ("<FlavorAccess flavor id: %s, tenant id: %s>" %
                    (flavor_access.flavor_id, flavor_access.tenant_id))

        for a in r:
            self.assertEqual(get_expected(a), repr(a))



# Copyright (c) 2014 VMware, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Server group interface.
"""

from novaclient import api_versions
from novaclient import base
from novaclient import exceptions
from novaclient.i18n import _


class ServerGroup(base.Resource):
    """
    A server group.
    """

    def __repr__(self):
        return '<ServerGroup: %s>' % self.id

    def delete(self):
        """
        Delete this server group.

        :returns: An instance of novaclient.base.TupleWithMeta
        """
        return self.manager.delete(self.id)


class ServerGroupsManager(base.ManagerWithFind):
    """
    Manage :class:`ServerGroup` resources.
    """
    resource_class = ServerGroup

    def list(self, all_projects=False, limit=None, offset=None):
        """Get a list of all server groups.

        :param all_projects: Lists server groups for all projects. (optional)
        :param limit: Maximum number of server groups to return. (optional)
                      Note the API server has a configurable default limit.
                      If no limit is specified here or limit is larger than
                      default, the default limit will be used.
        :param offset: Use with `limit` to return a slice of server
                       groups. `offset` is where to start in the groups
                       list. (optional)
        :returns: list of :class:`ServerGroup`.
        """
        qparams = {}
        if all_projects:
            qparams['all_projects'] = bool(all_projects)
        if limit:
            qparams['limit'] = int(limit)
        if offset:
            qparams['offset'] = int(offset)
        return self._list('/os-server-groups', 'server_groups',
                          filters=qparams)

    def get(self, id):
        """Get a specific server group.

        :param id: The ID of the :class:`ServerGroup` to get.
        :rtype: :class:`ServerGroup`
        """
        return self._get('/os-server-groups/%s' % id,
                         'server_group')

    def delete(self, id):
        """Delete a specific server group.

        :param id: The ID of the :class:`ServerGroup` to delete.
        :returns: An instance of novaclient.base.TupleWithMeta
        """
        return self._delete('/os-server-groups/%s' % id)

    @api_versions.wraps("2.0", "2.63")
    def create(self, name, policies):
        """Create (allocate) a server group.

        :param name: The name of the server group.
        :param policies: Policy name or a list of exactly one policy name to
            associate with the server group.
        :rtype: list of :class:`ServerGroup`
        """
        policies = policies if isinstance(policies, list) else [policies]
        body = {'server_group': {'name': name,
                                 'policies': policies}}
        return self._create('/os-server-groups', body, 'server_group')

    @api_versions.wraps("2.64")
    def create(self, name, policy, rules=None):
        """Create (allocate) a server group.

        :param name: The name of the server group.
        :param policy: Policy name to associate with the server group.
        :param rules: The rules of policy which is a dict, can be applied to
            the policy, now only ``max_server_per_host`` for ``anti-affinity``
            policy would be supported (optional).
        :rtype: list of :class:`ServerGroup`
        """
        body = {'server_group': {
            'name': name, 'policy': policy
        }}
        if rules:
            key = 'max_server_per_host'
            try:
                if key in rules:
                    rules[key] = int(rules[key])
            except ValueError:
                msg = _("Invalid '%(key)s' value: %(value)s")
                raise exceptions.CommandError(msg % {
                    'key': key, 'value': rules[key]})
            body['server_group']['rules'] = rules
        return self._create('/os-server-groups', body, 'server_group')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from novaclient.tests.functional import base


class TestUsageCLI(base.ClientTestBase):

    COMPUTE_API_VERSION = '2.1'

    def _get_num_servers_from_usage_output(self):
        output = self.nova('usage')
        servers = self._get_column_value_from_single_row_table(
            output, 'Servers')
        return int(servers)

    def _get_num_servers_by_tenant_from_usage_output(self):
        tenant_id = self._get_project_id(self.cli_clients.tenant_name)
        output = self.nova('usage --tenant=%s' % tenant_id)
        servers = self._get_column_value_from_single_row_table(
            output, 'Servers')
        return int(servers)

    def test_usage(self):
        before = self._get_num_servers_from_usage_output()
        self._create_server()
        after = self._get_num_servers_from_usage_output()
        self.assertGreater(after, before)

    def test_usage_tenant(self):
        before = self._get_num_servers_by_tenant_from_usage_output()
        self._create_server()
        after = self._get_num_servers_by_tenant_from_usage_output()
        self.assertGreater(after, before)


class TestUsageClient(base.ClientTestBase):

    COMPUTE_API_VERSION = '2.1'

    def _create_servers_in_time_window(self):
        start = datetime.datetime.now()
        self._create_server()
        self._create_server()
        end = datetime.datetime.now()
        return start, end

    def test_get(self):
        start, end = self._create_servers_in_time_window()
        tenant_id = self._get_project_id(self.cli_clients.tenant_name)
        usage = self.client.usage.get(tenant_id, start=start, end=end)
        self.assertEqual(tenant_id, usage.tenant_id)
        self.assertGreaterEqual(len(usage.server_usages), 2)

    def test_list(self):
        start, end = self._create_servers_in_time_window()
        tenant_id = self._get_project_id(self.cli_clients.tenant_name)
        usages = self.client.usage.list(start=start, end=end, detailed=True)
        tenant_ids = [usage.tenant_id for usage in usages]
        self.assertIn(tenant_id, tenant_ids)
        for usage in usages:
            if usage.tenant_id == tenant_id:
                self.assertGreaterEqual(len(usage.server_usages), 2)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io
import sys
from unittest import mock
from urllib import parse

from novaclient import base
from novaclient import exceptions
from novaclient.tests.unit import fakes
from novaclient.tests.unit import utils as test_utils
from novaclient import utils
from novaclient.v2 import servers

UUID = '8e8ec658-c7b0-4243-bdf8-6f7f2952c0d0'


class FakeResource(object):
    NAME_ATTR = 'name'

    request_ids = fakes.FAKE_REQUEST_ID_LIST

    def __init__(self, _id, properties):
        self.id = _id
        try:
            self.name = properties['name']
        except KeyError:
            pass

    def append_request_ids(self, resp):
        pass


class FakeManager(base.ManagerWithFind):

    resource_class = FakeResource

    resources = [
        FakeResource('1234', {'name': 'entity_one'}),
        FakeResource('12345', {'name': 'UPPER'}),
        FakeResource('123456', {'name': 'lower'}),
        FakeResource('1234567', {'name': 'Mixed'}),
        FakeResource('12345678', {'name': 'mixed'}),
        FakeResource(UUID, {'name': 'entity_two'}),
        FakeResource('5678', {'name': '9876'}),
        FakeResource('01234', {'name': 'entity_three'})
    ]

    is_alphanum_id_allowed = None

    def __init__(self, alphanum_id_allowed=False):
        self.is_alphanum_id_allowed = alphanum_id_allowed

    def get(self, resource_id):
        for resource in self.resources:
            if resource.id == str(resource_id):
                return resource
        raise exceptions.NotFound(resource_id)

    def list(self):
        return base.ListWithMeta(self.resources, fakes.FAKE_REQUEST_ID_LIST)


class FakeDisplayResource(object):
    NAME_ATTR = 'display_name'

    def __init__(self, _id, properties):
        self.id = _id
        try:
            self.display_name = properties['display_name']
        except KeyError:
            pass

    def append_request_ids(self, resp):
        pass


class FakeDisplayManager(FakeManager):

    resource_class = FakeDisplayResource

    resources = [
        FakeDisplayResource('4242', {'display_name': 'entity_three'}),
    ]


class FindResourceTestCase(test_utils.TestCase):

    def setUp(self):
        super(FindResourceTestCase, self).setUp()
        self.manager = FakeManager(None)

    def test_find_none(self):
        """Test a few non-valid inputs."""
        self.assertRaises(exceptions.CommandError,
                          utils.find_resource,
                          self.manager,
                          'asdf')
        self.assertRaises(exceptions.CommandError,
                          utils.find_resource,
                          self.manager,
                          None)
        self.assertRaises(exceptions.CommandError,
                          utils.find_resource,
                          self.manager,
                          {})

    def test_find_by_integer_id(self):
        output = utils.find_resource(self.manager, 1234)
        self.assertEqual(output, self.manager.get('1234'))

    def test_find_by_str_id(self):
        output = utils.find_resource(self.manager, '1234')
        self.assertEqual(output, self.manager.get('1234'))

    def test_find_by_uuid(self):
        output = utils.find_resource(self.manager, UUID)
        self.assertEqual(output, self.manager.get(UUID))

    def test_find_by_str_name(self):
        output = utils.find_resource(self.manager, 'entity_one')
        self.assertEqual(output, self.manager.get('1234'))

    def test_find_by_str_upper_name(self):
        output = utils.find_resource(self.manager, 'UPPER')
        self.assertEqual(output, self.manager.get('12345'))

    def test_find_by_str_lower_name(self):
        output = utils.find_resource(self.manager, 'lower')
        self.assertEqual(output, self.manager.get('123456'))

    def test_find_by_str_mix_name(self):
        output = utils.find_resource(self.manager, 'Mixed')
        self.assertEqual(output, self.manager.get('1234567'))

    def test_find_by_str_lower_name_mixed(self):
        output = utils.find_resource(self.manager, 'mixed')
        self.assertEqual(output, self.manager.get('12345678'))

    def test_find_by_str_display_name(self):
        display_manager = FakeDisplayManager(None)
        output = utils.find_resource(display_manager, 'entity_three')
        self.assertEqual(output, display_manager.get('4242'))

    def test_find_in_alphanum_allowed_manager_by_str_id_(self):
        alphanum_manager = FakeManager(True)
        output = utils.find_resource(alphanum_manager, '01234')
        self.assertEqual(output, alphanum_manager.get('01234'))

    def test_find_without_wrapping_exception(self):
        alphanum_manager = FakeManager(True)
        self.assertRaises(exceptions.NotFound, utils.find_resource,
                          alphanum_manager, 'not_exist', wrap_exception=False)
        res = alphanum_manager.resources[0]
        alphanum_manager.resources.append(res)
        self.assertRaises(exceptions.NoUniqueMatch, utils.find_resource,
                          alphanum_manager, res.name, wrap_exception=False)


class _FakeResult(object):
    def __init__(self, name, value):
        self.name = name
        self.value = value


class PrintResultTestCase(test_utils.TestCase):
    @mock.patch('sys.stdout', io.StringIO())
    def test_print_dict(self):
        dict = {'key': 'value'}
        utils.print_dict(dict)
        self.assertEqual('+----------+-------+\n'
                         '| Property | Value |\n'
                         '+----------+-------+\n'
                         '| key      | value |\n'
                         '+----------+-------+\n',
                         sys.stdout.getvalue())

    @mock.patch('sys.stdout', io.StringIO())
    def test_print_dict_wrap(self):
        dict = {'key1': 'not wrapped',
                'key2': 'this will be wrapped'}
        utils.print_dict(dict, wrap=16)
        self.assertEqual('+----------+--------------+\n'
                         '| Property | Value        |\n'
                         '+----------+--------------+\n'
                         '| key1     | not wrapped  |\n'
                         '| key2     | this will be |\n'
                         '|          | wrapped      |\n'
                         '+----------+--------------+\n',
                         sys.stdout.getvalue())

    @mock.patch('sys.stdout', io.StringIO())
    def test_print_list_sort_by_str(self):
        objs = [_FakeResult("k1", 1),
                _FakeResult("k3", 2),
                _FakeResult("k2", 3)]

        utils.print_list(objs, ["Name", "Value"], sortby_index=0)

        self.assertEqual('+------+-------+\n'
                         '| Name | Value |\n'
                         '+------+-------+\n'
                         '| k1   | 1     |\n'
                         '| k2   | 3     |\n'
                         '| k3   | 2     |\n'
                         '+------+-------+\n',
                         sys.stdout.getvalue())

    @mock.patch('sys.stdout', io.StringIO())
    def test_print_list_sort_by_integer(self):
        objs = [_FakeResult("k1", 1),
                _FakeResult("k3", 2),
                _FakeResult("k2", 3)]

        utils.print_list(objs, ["Name", "Value"], sortby_index=1)

        self.assertEqual('+------+-------+\n'
                         '| Name | Value |\n'
                         '+------+-------+\n'
                         '| k1   | 1     |\n'
                         '| k3   | 2     |\n'
                         '| k2   | 3     |\n'
                         '+------+-------+\n',
                         sys.stdout.getvalue())

    @mock.patch('sys.stdout', io.StringIO())
    def test_print_unicode_list(self):
        objs = [_FakeResult("k", '\u2026')]
        utils.print_list(objs, ["Name", "Value"])
        s = '\u2026'
        self.assertEqual('+------+-------+\n'
                         '| Name | Value |\n'
                         '+------+-------+\n'
                         '| k    | %s     |\n'
                         '+------+-------+\n' % s,
                         sys.stdout.getvalue())

    # without sorting
    @mock.patch('sys.stdout', io.StringIO())
    def test_print_list_sort_by_none(self):
        objs = [_FakeResult("k1", 1),
                _FakeResult("k3", 3),
                _FakeResult("k2", 2)]

        utils.print_list(objs, ["Name", "Value"], sortby_index=None)

        self.assertEqual('+------+-------+\n'
                         '| Name | Value |\n'
                         '+------+-------+\n'
                         '| k1   | 1     |\n'
                         '| k3   | 3     |\n'
                         '| k2   | 2     |\n'
                         '+------+-------+\n',
                         sys.stdout.getvalue())

    @mock.patch('sys.stdout', io.StringIO())
    def test_print_dict_dictionary(self):
        dict = {'k': {'foo': 'bar'}}
        utils.print_dict(dict)
        self.assertEqual('+----------+----------------+\n'
                         '| Property | Value          |\n'
                         '+----------+----------------+\n'
                         '| k        | {"foo": "bar"} |\n'
                         '+----------+----------------+\n',
                         sys.stdout.getvalue())

    @mock.patch('sys.stdout', io.StringIO())
    def test_print_dict_list_dictionary(self):
        dict = {'k': [{'foo': 'bar'}]}
        utils.print_dict(dict)
        self.assertEqual('+----------+------------------+\n'
                         '| Property | Value            |\n'
                         '+----------+------------------+\n'
                         '| k        | [{"foo": "bar"}] |\n'
                         '+----------+------------------+\n',
                         sys.stdout.getvalue())

    @mock.patch('sys.stdout', io.StringIO())
    def test_print_dict_list(self):
        dict = {'k': ['foo', 'bar']}
        utils.print_dict(dict)
        self.assertEqual('+----------+----------------+\n'
                         '| Property | Value          |\n'
                         '+----------+----------------+\n'
                         '| k        | ["foo", "bar"] |\n'
                         '+----------+----------------+\n',
                         sys.stdout.getvalue())

    @mock.patch('sys.stdout', io.StringIO())
    def test_print_large_dict_list(self):
        dict = {'k': ['foo1', 'bar1', 'foo2', 'bar2',
                      'foo3', 'bar3', 'foo4', 'bar4']}
        utils.print_dict(dict, wrap=40)
        self.assertEqual(
            '+----------+------------------------------------------+\n'
            '| Property | Value                                    |\n'
            '+----------+------------------------------------------+\n'
            '| k        | ["foo1", "bar1", "foo2", "bar2", "foo3", |\n'
            '|          | "bar3", "foo4", "bar4"]                  |\n'
            '+----------+------------------------------------------+\n',
            sys.stdout.getvalue())

    @mock.patch('sys.stdout', io.StringIO())
    def test_print_unicode_dict(self):
        dict = {'k': '\u2026'}
        utils.print_dict(dict)
        s = '\u2026'
        self.assertEqual('+----------+-------+\n'
                         '| Property | Value |\n'
                         '+----------+-------+\n'
                         '| k        | %s     |\n'
                         '+----------+-------+\n' % s,
                         sys.stdout.getvalue())


class FlattenTestCase(test_utils.TestCase):
    def test_flattening(self):
        squashed = utils.flatten_dict(
            {'a1': {'b1': 1234,
                    'b2': 'string',
                    'b3': set((1, 2, 3)),
                    'b4': {'c1': ['l', 'l', ['l']],
                           'c2': 'string'}},
             'a2': ['l'],
             'a3': ('t',),
             'a4': {}})

        self.assertEqual({'a1_b1': 1234,
                          'a1_b2': 'string',
                          'a1_b3': set([1, 2, 3]),
                          'a1_b4_c1': ['l', 'l', ['l']],
                          'a1_b4_c2': 'string',
                          'a2': ['l'],
                          'a3': ('t',),
                          'a4': {}},
                         squashed)

    def test_pretty_choice_dict(self):
        d = {}
        r = utils.pretty_choice_dict(d)
        self.assertEqual("", r)

        d = {"k1": "v1",
             "k2": "v2",
             "k3": "v3"}
        r = utils.pretty_choice_dict(d)
        self.assertEqual("'k1=v1', 'k2=v2', 'k3=v3'", r)


class ValidationsTestCase(test_utils.TestCase):
    def test_validate_flavor_metadata_keys_with_valid_keys(self):
        valid_keys = ['key1', 'month.price', 'I-Am:AK-ey.01-', 'spaces and _']
        utils.validate_flavor_metadata_keys(valid_keys)

    def test_validate_flavor_metadata_keys_with_invalid_keys(self):
        invalid_keys = ['/1', '?1', '%1', '<', '>', '\1']
        for key in invalid_keys:
            try:
                utils.validate_flavor_metadata_keys([key])
                self.fail("Invalid key passed validation: %s" % key)
            except exceptions.CommandError as ce:
                self.assertIn(key, str(ce))


class DoActionOnManyTestCase(test_utils.TestCase):

    def _test_do_action_on_many(self, side_effect, fail):
        action = mock.Mock(side_effect=side_effect)

        if fail:
            self.assertRaises(exceptions.CommandError,
                              utils.do_action_on_many,
                              action, [1, 2], 'success with %s', 'error')
        else:
            utils.do_action_on_many(action, [1, 2], 'success with %s', 'error')
        action.assert_has_calls([mock.call(1), mock.call(2)])

    def test_do_action_on_many_success(self):
        self._test_do_action_on_many([None, None], fail=False)

    def test_do_action_on_many_first_fails(self):
        self._test_do_action_on_many([Exception(), None], fail=True)

    def test_do_action_on_many_last_fails(self):
        self._test_do_action_on_many([None, Exception()], fail=True)

    @mock.patch('sys.stdout', new_callable=io.StringIO)
    def _test_do_action_on_many_resource_string(
            self, resource, expected_string, mock_stdout):
        utils.do_action_on_many(mock.Mock(), [resource], 'success with %s',
                                'error')
        self.assertIn('success with %s' % expected_string,
                      mock_stdout.getvalue())

    def test_do_action_on_many_resource_string_with_str(self):
        self._test_do_action_on_many_resource_string('resource1', 'resource1')

    def test_do_action_on_many_resource_string_with_human_id(self):
        resource = servers.Server(None, {'name': 'resource1'})
        self._test_do_action_on_many_resource_string(resource, 'resource1')

    def test_do_action_on_many_resource_string_with_id(self):
        resource = servers.Server(None, {'id': UUID})
        self._test_do_action_on_many_resource_string(resource, UUID)

    def test_do_action_on_many_resource_string_with_id_and_human_id(self):
        resource = servers.Server(None, {'name': 'resource1', 'id': UUID})
        self._test_do_action_on_many_resource_string(resource,
                                                     'resource1 (%s)' % UUID)


class RecordTimeTestCase(test_utils.TestCase):

    def test_record_time(self):
        times = []

        with utils.record_time(times, True, 'a', 'b'):
            pass
        self.assertEqual(1, len(times))
        self.assertEqual(3, len(times[0]))
        self.assertEqual('a b', times[0][0])
        self.assertIsInstance(times[0][1], float)
        self.assertIsInstance(times[0][2], float)

        times = []
        with utils.record_time(times, False, 'x'):
            pass
        self.assertEqual(0, len(times))


class PrepareQueryStringTestCase(test_utils.TestCase):

    def setUp(self):
        super(PrepareQueryStringTestCase, self).setUp()
        self.ustr = b'?\xd0\xbf=1&\xd1\x80=2'
        # in py3 real unicode symbols will be urlencoded
        self.ustr = self.ustr.decode('utf8')
        self.cases = (
            ({}, ''),
            (None, ''),
            ({'2': 2, '10': 1}, '?10=1&2=2'),
            ({'abc': 1, 'abc1': 2}, '?abc=1&abc1=2'),
            ({b'\xd0\xbf': 1, b'\xd1\x80': 2}, self.ustr),
            ({(1, 2): '1', (3, 4): '2'}, '?(1, 2)=1&(3, 4)=2')
        )

    def test_convert_dict_to_string(self):
        for case in self.cases:
            self.assertEqual(
                case[1],
                parse.unquote_plus(utils.prepare_query_string(case[0])))

    def test_get_url_with_filter(self):
        url = '/fake'
        for case in self.cases:
            self.assertEqual(
                '%s%s' % (url, case[1]),
                parse.unquote_plus(utils.get_url_with_filter(url, case[0])))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional import base


class TestOsServicesNovaClient(base.ClientTestBase):
    """Functional tests for os-services attributes"""

    COMPUTE_API_VERSION = "2.1"

    def test_os_services_list(self):
        table = self.nova('service-list')
        for serv in self.client.services.list():
            self.assertIn(serv.binary, table)

    def test_os_service_disable_enable(self):
        # Disable and enable Nova services in accordance with list of nova
        # services returned by client
        # NOTE(sdague): service disable has the chance in racing
        # with other tests. Now functional tests for novaclient are launched
        # in serial way (https://review.opendev.org/#/c/217768/), but
        # it's a potential issue for making these tests parallel in the future
        for serv in self.client.services.list():
            # In Pike the os-services API was made multi-cell aware and it
            # looks up services by host, which uses the host mapping record
            # in the API DB which is only populated for nova-compute services,
            # effectively making it impossible to perform actions like enable
            # or disable non-nova-compute services since the API won't be able
            # to find them. So filter out anything that's not nova-compute.
            if serv.binary != 'nova-compute':
                continue
            host = self._get_column_value_from_single_row_table(
                self.nova('service-list --binary %s' % serv.binary), 'Host')
            service = self.nova('service-disable %s' % host)
            self.addCleanup(self.nova, 'service-enable', params=host)
            status = self._get_column_value_from_single_row_table(
                service, 'Status')
            self.assertEqual('disabled', status)
            service = self.nova('service-enable %s' % host)
            status = self._get_column_value_from_single_row_table(
                service, 'Status')
            self.assertEqual('enabled', status)

    def test_os_service_disable_log_reason(self):
        for serv in self.client.services.list():
            # In Pike the os-services API was made multi-cell aware and it
            # looks up services by host, which uses the host mapping record
            # in the API DB which is only populated for nova-compute services,
            # effectively making it impossible to perform actions like enable
            # or disable non-nova-compute services since the API won't be able
            # to find them. So filter out anything that's not nova-compute.
            if serv.binary != 'nova-compute':
                continue
            host = self._get_column_value_from_single_row_table(
                self.nova('service-list --binary %s' % serv.binary), 'Host')
            service = self.nova(
                'service-disable --reason test_disable %s' % host)
            self.addCleanup(self.nova, 'service-enable', params=host)
            status = self._get_column_value_from_single_row_table(
                service, 'Status')
            log_reason = self._get_column_value_from_single_row_table(
                service, 'Disabled Reason')
            self.assertEqual('disabled', status)
            self.assertEqual('test_disable', log_reason)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock

from keystoneauth1 import session
from oslo_utils import uuidutils

import novaclient.api_versions
import novaclient.client
import novaclient.extension
from novaclient.tests.unit import utils
import novaclient.v2.client


class SessionClientTest(utils.TestCase):

    def test_timings(self):
        self.requests_mock.get('http://no.where')

        client = novaclient.client.SessionClient(session=session.Session())
        client.request("http://no.where", 'GET')
        self.assertEqual(0, len(client.times))

        client = novaclient.client.SessionClient(session=session.Session(),
                                                 timings=True)
        client.request("http://no.where", 'GET')
        self.assertEqual(1, len(client.times))
        self.assertEqual('GET http://no.where', client.times[0][0])

    def test_client_get_reset_timings_v2(self):
        cs = novaclient.client.SessionClient(session=session.Session())
        self.assertEqual(0, len(cs.get_timings()))
        cs.times.append("somevalue")
        self.assertEqual(1, len(cs.get_timings()))
        self.assertEqual("somevalue", cs.get_timings()[0])

        cs.reset_timings()
        self.assertEqual(0, len(cs.get_timings()))

    def test_global_id(self):
        global_id = "req-%s" % uuidutils.generate_uuid()
        self.requests_mock.get('http://no.where')

        client = novaclient.client.SessionClient(session=session.Session(),
                                                 global_request_id=global_id)
        client.request("http://no.where", 'GET')
        headers = self.requests_mock.last_request.headers
        self.assertEqual(headers['X-OpenStack-Request-ID'], global_id)


class ClientsUtilsTest(utils.TestCase):

    @mock.patch("novaclient.client._discover_via_entry_points")
    @mock.patch("novaclient.client._discover_via_python_path")
    @mock.patch("novaclient.extension.Extension")
    def test_discover_extensions_all(self, mock_extension,
                                     mock_discover_via_python_path,
                                     mock_discover_via_entry_points):
        def make_gen(start, end):
            def f(*args, **kwargs):
                for i in range(start, end):
                    yield "name-%s" % i, i
            return f

        mock_discover_via_python_path.side_effect = make_gen(0, 3)
        mock_discover_via_entry_points.side_effect = make_gen(3, 4)

        version = novaclient.api_versions.APIVersion("2.0")

        result = novaclient.client.discover_extensions(version)

        self.assertEqual([mock.call("name-%s" % i, i) for i in range(0, 4)],
                         mock_extension.call_args_list)
        mock_discover_via_python_path.assert_called_once_with()
        mock_discover_via_entry_points.assert_called_once_with()
        self.assertEqual([mock_extension()] * 4, result)

    @mock.patch("novaclient.client.warnings")
    def test__check_arguments(self, mock_warnings):
        release = "Coolest"

        # no reference
        novaclient.client._check_arguments({}, release=release,
                                           deprecated_name="foo")
        self.assertFalse(mock_warnings.warn.called)
        novaclient.client._check_arguments({}, release=release,
                                           deprecated_name="foo",
                                           right_name="bar")
        self.assertFalse(mock_warnings.warn.called)

        # with alternative
        original_kwargs = {"foo": "text"}
        actual_kwargs = copy.copy(original_kwargs)
        self.assertEqual(original_kwargs, actual_kwargs)
        novaclient.client._check_arguments(actual_kwargs, release=release,
                                           deprecated_name="foo",
                                           right_name="bar")
        self.assertNotEqual(original_kwargs, actual_kwargs)
        self.assertEqual({"bar": original_kwargs["foo"]}, actual_kwargs)
        self.assertTrue(mock_warnings.warn.called)

        mock_warnings.warn.reset_mock()

        # without alternative
        original_kwargs = {"foo": "text"}
        actual_kwargs = copy.copy(original_kwargs)
        self.assertEqual(original_kwargs, actual_kwargs)
        novaclient.client._check_arguments(actual_kwargs, release=release,
                                           deprecated_name="foo")
        self.assertNotEqual(original_kwargs, actual_kwargs)
        self.assertEqual({}, actual_kwargs)
        self.assertTrue(mock_warnings.warn.called)


class ClientTest(utils.TestCase):

    def test_logger(self):
        client = novaclient.client.Client('2.1', logger=mock.sentinel.logger)
        self.assertEqual(mock.sentinel.logger, client.logger)
        self.assertEqual(mock.sentinel.logger, client.client.logger)
        client = novaclient.client.Client('2.1')
        self.assertEqual('novaclient.v2.client', client.logger.name)
        self.assertIsNotNone(client.client.logger)
        self.assertEqual('novaclient.v2.client', client.client.logger.name)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional import base


class ResourceChecker(base.ClientTestBase):

    def runTest(self):
        pass

    def check(self):
        self.setUp()

        print("$ nova list --all-tenants")
        print(self.nova("list", params="--all-tenants"))
        print("\n")


if __name__ == "__main__":
    ResourceChecker().check()



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from novaclient import api_versions
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import usage


class UsageTest(utils.TestCase):
    def setUp(self):
        super(UsageTest, self).setUp()
        self.cs = fakes.FakeClient(api_versions.APIVersion("2.0"))
        self.usage_type = self._get_usage_type()

    def _get_usage_type(self):
        return usage.Usage

    def test_usage_list(self, detailed=False):
        now = datetime.datetime.now()
        usages = self.cs.usage.list(now, now, detailed)
        self.assert_request_id(usages, fakes.FAKE_REQUEST_ID_LIST)

        self.cs.assert_called(
            'GET',
            "/os-simple-tenant-usage?" +
            ("start=%s&" % now.isoformat()) +
            ("end=%s&" % now.isoformat()) +
            ("detailed=%s" % int(bool(detailed))))
        for u in usages:
            self.assertIsInstance(u, usage.Usage)

    def test_usage_list_detailed(self):
        self.test_usage_list(True)

    def test_usage_get(self):
        now = datetime.datetime.now()
        u = self.cs.usage.get("tenantfoo", now, now)
        self.assert_request_id(u, fakes.FAKE_REQUEST_ID_LIST)

        self.cs.assert_called(
            'GET',
            "/os-simple-tenant-usage/tenantfoo?" +
            ("start=%s&" % now.isoformat()) +
            ("end=%s" % now.isoformat()))
        self.assertIsInstance(u, usage.Usage)

    def test_usage_class_get(self):
        start = '2012-01-22T19:48:41.750722'
        stop = '2012-01-22T19:48:41.750722'

        info = {'tenant_id': 'tenantfoo', 'start': start,
                'stop': stop}
        u = usage.Usage(self.cs.usage, info)
        u.get()
        self.assert_request_id(u, fakes.FAKE_REQUEST_ID_LIST)

        self.cs.assert_called(
            'GET',
            "/os-simple-tenant-usage/tenantfoo?start=%s&end=%s" %
            (start, stop))


class UsageV40Test(UsageTest):
    def setUp(self):
        super(UsageV40Test, self).setUp()
        self.cs.api_version = api_versions.APIVersion('2.40')

    def test_usage_list_with_paging(self):
        now = datetime.datetime.now()
        usages = self.cs.usage.list(now, now, marker='some-uuid', limit=3)
        self.assert_request_id(usages, fakes.FAKE_REQUEST_ID_LIST)

        self.cs.assert_called(
            'GET',
            '/os-simple-tenant-usage?' +
            ('start=%s&' % now.isoformat()) +
            ('end=%s&' % now.isoformat()) +
            ('limit=3&marker=some-uuid&detailed=0'))
        for u in usages:
            self.assertIsInstance(u, usage.Usage)

    def test_usage_list_detailed_with_paging(self):
        now = datetime.datetime.now()
        usages = self.cs.usage.list(
            now, now, detailed=True, marker='some-uuid', limit=3)
        self.assert_request_id(usages, fakes.FAKE_REQUEST_ID_LIST)

        self.cs.assert_called(
            'GET',
            '/os-simple-tenant-usage?' +
            ('start=%s&' % now.isoformat()) +
            ('end=%s&' % now.isoformat()) +
            ('limit=3&marker=some-uuid&detailed=1'))
        for u in usages:
            self.assertIsInstance(u, usage.Usage)

    def test_usage_get_with_paging(self):
        now = datetime.datetime.now()
        u = self.cs.usage.get(
            'tenantfoo', now, now, marker='some-uuid', limit=3)
        self.assert_request_id(u, fakes.FAKE_REQUEST_ID_LIST)

        self.cs.assert_called(
            'GET',
            '/os-simple-tenant-usage/tenantfoo?' +
            ('start=%s&' % now.isoformat()) +
            ('end=%s&' % now.isoformat()) +
            ('limit=3&marker=some-uuid'))
        self.assertIsInstance(u, usage.Usage)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from oslo_utils import timeutils
from oslo_utils import uuidutils
from tempest.lib import exceptions

from novaclient.tests.functional import base


class TestInstanceActionCLI(base.ClientTestBase):

    COMPUTE_API_VERSION = "2.21"

    def _test_cmd_with_not_existing_instance(self, cmd, args):
        try:
            self.nova("%s %s" % (cmd, args))
        except exceptions.CommandFailed as e:
            self.assertIn("ERROR (NotFound):", str(e))
        else:
            self.fail("%s is not failed on non existing instance." % cmd)

    def test_show_action_with_not_existing_instance(self):
        name_or_uuid = uuidutils.generate_uuid()
        request_id = uuidutils.generate_uuid()
        self._test_cmd_with_not_existing_instance(
            "instance-action", "%s %s" % (name_or_uuid, request_id))

    def test_list_actions_with_not_existing_instance(self):
        name_or_uuid = uuidutils.generate_uuid()
        self._test_cmd_with_not_existing_instance("instance-action-list",
                                                  name_or_uuid)

    def test_show_and_list_actions_on_deleted_instance(self):
        server = self._create_server(add_cleanup=False)
        server.delete()
        self.wait_for_resource_delete(server, self.client.servers)

        output = self.nova("instance-action-list %s" % server.id)
        # NOTE(andreykurilin): output is not a single row table, so we can
        #   obtain just "create" action. It should be enough for testing
        #   "nova instance-action <server> <request-id>" command
        request_id = self._get_column_value_from_single_row_table(
            output, "Request_ID")

        output = self.nova("instance-action %s %s" % (server.id, request_id))

        # ensure that obtained action is "create".
        self.assertEqual("create",
                         self._get_value_from_the_table(output, "action"))


class TestInstanceActionCLIV258(TestInstanceActionCLI):
    """Instance action functional tests for v2.58 nova-api microversion."""

    COMPUTE_API_VERSION = "2.58"
    # Does this microversion return a hostId field in the event response?
    expect_event_hostId_field = False

    def test_list_instance_action_with_marker_and_limit(self):
        server = self._create_server()
        server.stop()
        # The actions are sorted by created_at in descending order,
        # and now we have two actions: create and stop.
        output = self.nova("instance-action-list %s --limit 1" % server.id)
        marker_req = self._get_column_value_from_single_row_table(
            output, "Request_ID")
        action = self._get_list_of_values_from_single_column_table(
            output, "Action")
        # The stop action was most recently created so it's what
        # we get back when limit=1.
        self.assertEqual(action, ['stop'])

        output = self.nova("instance-action-list %s --limit 1 "
                           "--marker %s" % (server.id, marker_req))
        action = self._get_list_of_values_from_single_column_table(
            output, "Action")
        self.assertEqual(action, ['create'])
        if not self.expect_event_hostId_field:
            # Make sure host and hostId are not in the response when
            # microversion is less than 2.62.
            output = self.nova("instance-action %s %s" % (
                server.id, marker_req))
            self.assertNotIn("'host'", output)
            self.assertNotIn("'hostId'", output)

    def test_list_instance_action_with_changes_since(self):
        # Ignore microseconds to make this a deterministic test.
        before_create = timeutils.utcnow().replace(microsecond=0).isoformat()
        server = self._create_server()
        time.sleep(2)
        before_stop = timeutils.utcnow().replace(microsecond=0).isoformat()
        server.stop()

        create_output = self.nova(
            "instance-action-list %s --changes-since %s" %
            (server.id, before_create))
        action = self._get_list_of_values_from_single_column_table(
            create_output, "Action")
        # The actions are sorted by created_at in descending order.
        self.assertEqual(action, ['create', 'stop'])

        stop_output = self.nova("instance-action-list %s --changes-since %s" %
                                (server.id, before_stop))
        action = self._get_list_of_values_from_single_column_table(
            stop_output, "Action")
        # Provide detailed debug information if this fails.
        self.assertEqual(action, ['stop'],
                         'Expected to find the stop action with '
                         '--changes-since=%s but got: %s\n\n'
                         'First instance-action-list output: %s' %
                         (before_stop, stop_output, create_output))


class TestInstanceActionCLIV262(TestInstanceActionCLIV258,
                                base.ProjectTestBase):
    """Instance action functional tests for v2.62 nova-api microversion."""

    COMPUTE_API_VERSION = "2.62"
    expect_event_hostId_field = True

    def test_show_actions_with_host(self):
        name = self.name_generate()
        # Create server with non-admin user
        server = self.another_nova('boot --flavor %s --image %s --poll %s' %
                                   (self.flavor.name, self.image.name, name))
        server_id = self._get_value_from_the_table(server, 'id')
        self.addCleanup(self.client.servers.delete, server_id)

        output = self.nova("instance-action-list %s" % server_id)
        request_id = self._get_column_value_from_single_row_table(
            output, "Request_ID")

        # Only the 'hostId' are exposed to non-admin
        output = self.another_nova(
            "instance-action %s %s" % (server_id, request_id))
        self.assertNotIn("'host'", output)
        self.assertIn("'hostId'", output)

        # The 'host' and 'hostId' are exposed to admin
        output = self.nova("instance-action %s %s" % (server_id, request_id))
        self.assertIn("'host'", output)
        self.assertIn("'hostId'", output)


class TestInstanceActionCLIV266(TestInstanceActionCLIV258,
                                base.ProjectTestBase):
    """Instance action functional tests for v2.66 nova-api microversion."""

    COMPUTE_API_VERSION = "2.66"
    expect_event_hostId_field = True

    def _wait_for_instance_actions(self, server, expected_num_of_actions):
        start_time = time.time()
        # Time out after 60 seconds
        while time.time() - start_time < 60:
            actions = self.client.instance_action.list(server)
            if len(actions) == expected_num_of_actions:
                break
            # Sleep 1 second
            time.sleep(1)
        else:
            self.fail("The number of instance actions for server %s "
                      "was not %d after 60 s" %
                      (server.id, expected_num_of_actions))
        # NOTE(takashin): In some DBMSs (e.g. MySQL 5.7), fractions
        # (millisecond and microsecond) of DateTime column is not stored
        # by default. So sleep an extra second.
        time.sleep(1)
        # Return time
        return timeutils.utcnow().isoformat()

    def test_list_instance_action_with_changes_before(self):
        server = self._create_server()
        end_create = self._wait_for_instance_actions(server, 1)
        # NOTE(takashin): In some DBMSs (e.g. MySQL 5.7), fractions
        # (millisecond and microsecond) of DateTime column is not stored
        # by default. So sleep a second.
        time.sleep(1)
        server.stop()
        end_stop = self._wait_for_instance_actions(server, 2)

        stop_output = self.nova(
            "instance-action-list %s --changes-before %s" %
            (server.id, end_stop))
        action = self._get_list_of_values_from_single_column_table(
            stop_output, "Action")
        # The actions are sorted by created_at in descending order.
        self.assertEqual(['create', 'stop'], action,
                         'Expected to find the create and stop actions with '
                         '--changes-before=%s but got: %s\n\n' %
                         (end_stop, stop_output))

        create_output = self.nova(
            "instance-action-list %s --changes-before %s" %
            (server.id, end_create))
        action = self._get_list_of_values_from_single_column_table(
            create_output, "Action")
        # Provide detailed debug information if this fails.
        self.assertEqual(['create'], action,
                         'Expected to find the create action with '
                         '--changes-before=%s but got: %s\n\n'
                         'First instance-action-list output: %s' %
                         (end_create, create_output, stop_output))



# Copyright 2010 Jacob Kaplan-Moss

# Copyright 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Base utilities to build API operation managers and objects on top of.
"""

import abc
import contextlib
import copy
import hashlib
import os
import threading

from oslo_utils import reflection
from oslo_utils import strutils
import requests

from novaclient import exceptions
from novaclient import utils


def getid(obj):
    """Get object's ID or object.

    Abstracts the common pattern of allowing both an object or an object's ID
    as a parameter when dealing with relationships.
    """
    return getattr(obj, 'id', obj)


# TODO(aababilov): call run_hooks() in HookableMixin's child classes
class HookableMixin(object):
    """Mixin so classes can register and run hooks."""
    _hooks_map = {}

    @classmethod
    def add_hook(cls, hook_type, hook_func):
        """Add a new hook of specified type.

        :param cls: class that registers hooks
        :param hook_type: hook type, e.g., '__pre_parse_args__'
        :param hook_func: hook function
        """
        if hook_type not in cls._hooks_map:
            cls._hooks_map[hook_type] = []

        cls._hooks_map[hook_type].append(hook_func)

    @classmethod
    def run_hooks(cls, hook_type, *args, **kwargs):
        """Run all hooks of specified type.

        :param cls: class that registers hooks
        :param hook_type: hook type, e.g., '__pre_parse_args__'
        :param args: args to be passed to every hook function
        :param kwargs: kwargs to be passed to every hook function
        """
        hook_funcs = cls._hooks_map.get(hook_type) or []
        for hook_func in hook_funcs:
            hook_func(*args, **kwargs)


class RequestIdMixin(object):
    """Wrapper class to expose x-openstack-request-id to the caller.
    """
    def request_ids_setup(self):
        self.x_openstack_request_ids = []

    @property
    def request_ids(self):
        return self.x_openstack_request_ids

    def append_request_ids(self, resp):
        """Add request_ids as an attribute to the object

        :param resp: Response object or list of Response objects
        """
        if isinstance(resp, list):
            # Add list of request_ids if response is of type list.
            for resp_obj in resp:
                self._append_request_id(resp_obj)
        elif resp is not None:
            # Add request_ids if response contains single object.
            self._append_request_id(resp)

    def _append_request_id(self, resp):
        if isinstance(resp, requests.Response):
            # Extract 'x-openstack-request-id' from headers if
            # response is a Response object.
            request_id = (resp.headers.get('x-openstack-request-id') or
                          resp.headers.get('x-compute-request-id'))
        else:
            # If resp is of type string or None.
            request_id = resp
        if request_id not in self.x_openstack_request_ids:
            self.x_openstack_request_ids.append(request_id)


class Resource(RequestIdMixin):
    """Base class for OpenStack resources (tenant, user, etc.).

    This is pretty much just a bag for attributes.
    """

    HUMAN_ID = False
    NAME_ATTR = 'name'

    def __init__(self, manager, info, loaded=False, resp=None):
        """Populate and bind to a manager.

        :param manager: BaseManager object
        :param info: dictionary representing resource attributes
        :param loaded: prevent lazy-loading if set to True
        :param resp: Response or list of Response objects
        """
        self.manager = manager
        self._info = info
        self._add_details(info)
        self._loaded = loaded
        self.request_ids_setup()
        self.append_request_ids(resp)

    def __repr__(self):
        reprkeys = sorted(k
                          for k in self.__dict__.keys()
                          if k[0] != '_' and
                          k not in ['manager', 'x_openstack_request_ids'])
        info = ", ".join("%s=%s" % (k, getattr(self, k)) for k in reprkeys)
        return "<%s %s>" % (self.__class__.__name__, info)

    @property
    def api_version(self):
        return self.manager.api_version

    @property
    def human_id(self):
        """Human-readable ID which can be used for bash completion.
        """
        if self.HUMAN_ID:
            name = getattr(self, self.NAME_ATTR, None)
            if name is not None:
                return strutils.to_slug(name)
        return None

    def _add_details(self, info):
        for (k, v) in info.items():
            try:
                setattr(self, k, v)
                self._info[k] = v
            except AttributeError:
                # In this case we already defined the attribute on the class
                pass

    def __getattr__(self, k):
        if k not in self.__dict__:
            # NOTE(bcwaldon): disallow lazy-loading if already loaded once
            if not self.is_loaded():
                self.get()
                return self.__getattr__(k)

            raise AttributeError(k)
        else:
            return self.__dict__[k]

    def get(self):
        """Support for lazy loading details.

        Some clients, such as novaclient have the option to lazy load the
        details, details which can be loaded with this function.
        """
        # set_loaded() first ... so if we have to bail, we know we tried.
        self.set_loaded(True)
        if not hasattr(self.manager, 'get'):
            return

        new = self.manager.get(self.id)
        if new:
            self._add_details(new._info)
            # The 'request_ids' attribute has been added,
            # so store the request id to it instead of _info
            self.append_request_ids(new.request_ids)

    def __eq__(self, other):
        if not isinstance(other, Resource):
            return NotImplemented
        # two resources of different types are not equal
        if not isinstance(other, self.__class__):
            return False
        if hasattr(self, 'id') and hasattr(other, 'id'):
            return self.id == other.id
        return self._info == other._info

    def __ne__(self, other):
        # Using not of '==' implementation because the not of
        # __eq__, when it returns NotImplemented, is returning False.
        return not self == other

    def is_loaded(self):
        return self._loaded

    def set_loaded(self, val):
        self._loaded = val

    def set_info(self, key, value):
        self._info[key] = value

    def to_dict(self):
        return copy.deepcopy(self._info)


class Manager(HookableMixin):
    """Manager for API service.

    Managers interact with a particular type of API (servers, flavors, images,
    etc.) and provide CRUD operations for them.
    """
    resource_class = None
    cache_lock = threading.RLock()

    def __init__(self, api):
        self.api = api

    @property
    def client(self):
        return self.api.client

    @property
    def api_version(self):
        return self.api.api_version

    def _list(self, url, response_key, obj_class=None, body=None,
              filters=None):
        if filters:
            url = utils.get_url_with_filter(url, filters)
        if body:
            resp, body = self.api.client.post(url, body=body)
        else:
            resp, body = self.api.client.get(url)

        if obj_class is None:
            obj_class = self.resource_class

        data = body[response_key]
        # NOTE(ja): keystone returns values as list as {'values': [ ... ]}
        #           unlike other services which just return the list...
        if isinstance(data, dict):
            try:
                data = data['values']
            except KeyError:
                pass

        with self.completion_cache('human_id', obj_class, mode="w"):
            with self.completion_cache('uuid', obj_class, mode="w"):
                items = [obj_class(self, res, loaded=True)
                         for res in data if res]
                return ListWithMeta(items, resp)

    @contextlib.contextmanager
    def alternate_service_type(self, default, allowed_types=()):
        original_service_type = self.api.client.service_type
        if original_service_type in allowed_types:
            yield
        else:
            self.api.client.service_type = default
            try:
                yield
            finally:
                self.api.client.service_type = original_service_type

    @contextlib.contextmanager
    def completion_cache(self, cache_type, obj_class, mode):
        """The completion cache for bash autocompletion.

        The completion cache store items that can be used for bash
        autocompletion, like UUIDs or human-friendly IDs.

        A resource listing will clear and repopulate the cache.

        A resource create will append to the cache.

        Delete is not handled because listings are assumed to be performed
        often enough to keep the cache reasonably up-to-date.
        """
        # NOTE(wryan): This lock protects read and write access to the
        # completion caches
        with self.cache_lock:
            base_dir = utils.env('NOVACLIENT_UUID_CACHE_DIR',
                                 default="~/.novaclient")

            # NOTE(sirp): Keep separate UUID caches for each username +
            # endpoint pair
            username = utils.env('OS_USERNAME', 'NOVA_USERNAME')
            url = utils.env('OS_URL', 'NOVA_URL')
            uniqifier = hashlib.sha256(username.encode('utf-8') +
                                       url.encode('utf-8')).hexdigest()

            cache_dir = os.path.expanduser(os.path.join(base_dir, uniqifier))

            try:
                os.makedirs(cache_dir, 0o755)
            except OSError:
                # NOTE(kiall): This is typically either permission denied while
                #              attempting to create the directory, or the
                #              directory already exists. Either way, don't
                #              fail.
                pass

            resource = obj_class.__name__.lower()
            filename = "%s-%s-cache" % (resource, cache_type.replace('_', '-'))
            path = os.path.join(cache_dir, filename)

            cache_attr = "_%s_cache" % cache_type

            try:
                setattr(self, cache_attr, open(path, mode))
            except IOError:
                # NOTE(kiall): This is typically a permission denied while
                #              attempting to write the cache file.
                pass

            try:
                yield
            finally:
                cache = getattr(self, cache_attr, None)
                if cache:
                    cache.close()
                    delattr(self, cache_attr)

    def write_to_completion_cache(self, cache_type, val):
        cache = getattr(self, "_%s_cache" % cache_type, None)
        if cache:
            cache.write("%s\n" % val)

    def _get(self, url, response_key, filters=None):
        if filters:
            url = utils.get_url_with_filter(url, filters)
        resp, body = self.api.client.get(url)
        if response_key is not None:
            content = body[response_key]
        else:
            content = body
        return self.resource_class(self, content, loaded=True,
                                   resp=resp)

    def _create(self, url, body, response_key, return_raw=False,
                obj_class=None, **kwargs):
        self.run_hooks('modify_body_for_create', body, **kwargs)
        resp, body = self.api.client.post(url, body=body)
        if return_raw:
            return self.convert_into_with_meta(body[response_key], resp)

        if obj_class is None:
            obj_class = self.resource_class

        with self.completion_cache('human_id', obj_class, mode="a"):
            with self.completion_cache('uuid', obj_class, mode="a"):
                return obj_class(self, body[response_key], resp=resp)

    def _delete(self, url):
        resp, body = self.api.client.delete(url)
        return self.convert_into_with_meta(body, resp)

    def _update(self, url, body, response_key=None, **kwargs):
        self.run_hooks('modify_body_for_update', body, **kwargs)
        resp, body = self.api.client.put(url, body=body)
        if body:
            if response_key:
                return self.resource_class(self, body[response_key], resp=resp)
            else:
                return self.resource_class(self, body, resp=resp)
        else:
            return StrWithMeta(body, resp)

    def convert_into_with_meta(self, item, resp):
        if isinstance(item, str):
            return StrWithMeta(item, resp)
        elif isinstance(item, bytes):
            return BytesWithMeta(item, resp)
        elif isinstance(item, list):
            return ListWithMeta(item, resp)
        elif isinstance(item, tuple):
            return TupleWithMeta(item, resp)
        elif item is None:
            return TupleWithMeta((), resp)
        else:
            return DictWithMeta(item, resp)


class ManagerWithFind(Manager, metaclass=abc.ABCMeta):
    """Like a `Manager`, but with additional `find()`/`findall()` methods."""

    @abc.abstractmethod
    def list(self):
        pass

    def find(self, **kwargs):
        """Find a single item with attributes matching ``**kwargs``."""
        matches = self.findall(**kwargs)
        num_matches = len(matches)
        if num_matches == 0:
            msg = "No %s matching %s." % (self.resource_class.__name__, kwargs)
            raise exceptions.NotFound(404, msg)
        elif num_matches > 1:
            raise exceptions.NoUniqueMatch
        else:
            matches[0].append_request_ids(matches.request_ids)
            return matches[0]

    def findall(self, **kwargs):
        """Find all items with attributes matching ``**kwargs``."""
        found = ListWithMeta([], None)
        searches = kwargs.items()

        detailed = True
        list_kwargs = {}

        list_argspec = reflection.get_callable_args(self.list)
        if 'detailed' in list_argspec:
            detailed = ("human_id" not in kwargs and
                        "name" not in kwargs and
                        "display_name" not in kwargs)
            list_kwargs['detailed'] = detailed

        if 'is_public' in list_argspec and 'is_public' in kwargs:
            is_public = kwargs['is_public']
            list_kwargs['is_public'] = is_public
            if is_public is None:
                tmp_kwargs = kwargs.copy()
                del tmp_kwargs['is_public']
                searches = tmp_kwargs.items()

        if 'search_opts' in list_argspec:
            # pass search_opts in to do server side based filtering.
            # TODO(jogo) not all search_opts support regex, find way to
            # identify when to use regex and when to use string matching.
            # volumes does not support regex while servers does. So when
            # doing findall on servers some client side filtering is still
            # needed.
            if "human_id" in kwargs:
                list_kwargs['search_opts'] = {"name": kwargs["human_id"]}
            elif "name" in kwargs:
                list_kwargs['search_opts'] = {"name": kwargs["name"]}
            elif "display_name" in kwargs:
                list_kwargs['search_opts'] = {"name": kwargs["display_name"]}
            if "all_tenants" in kwargs:
                all_tenants = kwargs['all_tenants']
                list_kwargs['search_opts']['all_tenants'] = all_tenants
                searches = [(k, v) for k, v in searches if k != 'all_tenants']
            if "deleted" in kwargs:
                deleted = kwargs['deleted']
                list_kwargs['search_opts']['deleted'] = deleted
                searches = [(k, v) for k, v in searches if k != 'deleted']

        listing = self.list(**list_kwargs)
        found.append_request_ids(listing.request_ids)

        for obj in listing:
            try:
                if all(getattr(obj, attr) == value
                        for (attr, value) in searches):
                    if detailed:
                        found.append(obj)
                    else:
                        detail = self.get(obj.id)
                        found.append(detail)
                        found.append_request_ids(detail.request_ids)
            except AttributeError:
                continue

        return found


class BootingManagerWithFind(ManagerWithFind):
    """Like a `ManagerWithFind`, but has the ability to boot servers."""

    def _parse_block_device_mapping(self, block_device_mapping):
        """Parses legacy block device mapping."""
        # FIXME(andreykurilin): make it work with block device mapping v2

        bdm = []

        for device_name, mapping in block_device_mapping.items():
            #
            # The mapping is in the format:
            # <id>:[<type>]:[<size(GiB)>]:[<delete_on_terminate>]
            #
            bdm_dict = {'device_name': device_name}

            mapping_parts = mapping.split(':')
            source_id = mapping_parts[0]

            if len(mapping_parts) == 1:
                bdm_dict['volume_id'] = source_id
            elif len(mapping_parts) > 1:
                source_type = mapping_parts[1]
                if source_type.startswith('snap'):
                    bdm_dict['snapshot_id'] = source_id
                else:
                    bdm_dict['volume_id'] = source_id

            if len(mapping_parts) > 2 and mapping_parts[2]:
                bdm_dict['volume_size'] = str(int(mapping_parts[2]))

            if len(mapping_parts) > 3:
                bdm_dict['delete_on_termination'] = mapping_parts[3]

            bdm.append(bdm_dict)
        return bdm


class ListWithMeta(list, RequestIdMixin):
    def __init__(self, values, resp):
        super(ListWithMeta, self).__init__(values)
        self.request_ids_setup()
        self.append_request_ids(resp)


class DictWithMeta(dict, RequestIdMixin):
    def __init__(self, values, resp):
        super(DictWithMeta, self).__init__(values)
        self.request_ids_setup()
        self.append_request_ids(resp)


class TupleWithMeta(tuple, RequestIdMixin):
    def __new__(cls, values, resp):
        return super(TupleWithMeta, cls).__new__(cls, values)

    def __init__(self, values, resp):
        self.request_ids_setup()
        self.append_request_ids(resp)


class StrWithMeta(str, RequestIdMixin):
    def __new__(cls, value, resp):
        return super(StrWithMeta, cls).__new__(cls, value)

    def __init__(self, values, resp):
        self.request_ids_setup()
        self.append_request_ids(resp)


class BytesWithMeta(bytes, RequestIdMixin):
    def __new__(cls, value, resp):
        return super(BytesWithMeta, cls).__new__(cls, value)

    def __init__(self, values, resp):
        self.request_ids_setup()
        self.append_request_ids(resp)



# Copyright (C) 2013, Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Assisted volume snapshots - to be used by Cinder and not end users.
"""

from novaclient import api_versions
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes


class AssistedVolumeSnapshotsTestCase(utils.TestCase):
    def setUp(self):
        super(AssistedVolumeSnapshotsTestCase, self).setUp()
        self.cs = fakes.FakeClient(api_versions.APIVersion("2.1"))

    def test_create_snap(self):
        vs = self.cs.assisted_volume_snapshots.create('1', {})
        self.assert_request_id(vs, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('POST', '/os-assisted-volume-snapshots')

    def test_delete_snap(self):
        vs = self.cs.assisted_volume_snapshots.delete('x', {})
        self.assert_request_id(vs, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called(
            'DELETE',
            '/os-assisted-volume-snapshots/x?delete_info={}')



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.unit.fixture_data import base


class Fixture(base.Fixture):
    base_url = 'servers'

    def setUp(self):
        super(Fixture, self).setUp()
        url = self.url('1234', 'migrations', '1', 'action')
        self.requests_mock.post(url,
                                status_code=202,
                                headers=self.json_headers)

        get_migrations = {'migrations': [
            {
                "created_at": "2016-01-29T13:42:02.000000",
                "dest_compute": "compute2",
                "dest_host": "1.2.3.4",
                "dest_node": "node2",
                "id": 1,
                "server_uuid": "4cfba335-03d8-49b2-8c52-e69043d1e8fe",
                "source_compute": "compute1",
                "source_node": "node1",
                "status": "running",
                "memory_total_bytes": 123456,
                "memory_processed_bytes": 12345,
                "memory_remaining_bytes": 120000,
                "disk_total_bytes": 234567,
                "disk_processed_bytes": 23456,
                "disk_remaining_bytes": 230000,
                "updated_at": "2016-01-29T13:42:02.000000"
            }]}

        url = self.url('1234', 'migrations')
        self.requests_mock.get(url,
                               json=get_migrations,
                               headers=self.json_headers)

        get_migration = {'migration': {
            "created_at": "2016-01-29T13:42:02.000000",
            "dest_compute": "compute2",
            "dest_host": "1.2.3.4",
            "dest_node": "node2",
            "id": 1,
            "server_uuid": "4cfba335-03d8-49b2-8c52-e69043d1e8fe",
            "source_compute": "compute1",
            "source_node": "node1",
            "status": "running",
            "memory_total_bytes": 123456,
            "memory_processed_bytes": 12345,
            "memory_remaining_bytes": 120000,
            "disk_total_bytes": 234567,
            "disk_processed_bytes": 23456,
            "disk_remaining_bytes": 230000,
            "updated_at": "2016-01-29T13:42:02.000000"
        }}

        url = self.url('1234', 'migrations', '1')
        self.requests_mock.get(url,
                               json=get_migration,
                               headers=self.json_headers)
        url = self.url('1234', 'migrations', '1')
        self.requests_mock.delete(url,
                                  status_code=202,
                                  headers=self.json_headers)



# Copyright (c) 2013, OpenStack
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from novaclient import api_versions
from novaclient import base
from novaclient import exceptions
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import flavors


class FlavorsTest(utils.TestCase):
    def setUp(self):
        super(FlavorsTest, self).setUp()
        self.cs = fakes.FakeClient(api_versions.APIVersion("2.0"))
        self.flavor_type = self._get_flavor_type()

    def _get_flavor_type(self):
        return flavors.Flavor

    def test_list_flavors(self):
        fl = self.cs.flavors.list()
        self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/detail')
        for flavor in fl:
            self.assertIsInstance(flavor, self.flavor_type)

    def test_list_flavors_undetailed(self):
        fl = self.cs.flavors.list(detailed=False)
        self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors')
        for flavor in fl:
            self.assertIsInstance(flavor, self.flavor_type)

    def test_list_flavors_with_marker_limit(self):
        fl = self.cs.flavors.list(marker=1234, limit=4)
        self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/detail?limit=4&marker=1234')

    def test_list_flavors_with_min_disk(self):
        fl = self.cs.flavors.list(min_disk=20)
        self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/detail?minDisk=20')

    def test_list_flavors_with_min_ram(self):
        fl = self.cs.flavors.list(min_ram=512)
        self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/detail?minRam=512')

    def test_list_flavors_with_sort_key_dir(self):
        fl = self.cs.flavors.list(sort_key='id', sort_dir='asc')
        self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET',
                              '/flavors/detail?sort_dir=asc&sort_key=id')

    def test_list_flavors_is_public_none(self):
        fl = self.cs.flavors.list(is_public=None)
        self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/detail?is_public=None')
        for flavor in fl:
            self.assertIsInstance(flavor, self.flavor_type)

    def test_list_flavors_is_public_false(self):
        fl = self.cs.flavors.list(is_public=False)
        self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/detail?is_public=False')
        for flavor in fl:
            self.assertIsInstance(flavor, self.flavor_type)

    def test_list_flavors_is_public_true(self):
        fl = self.cs.flavors.list(is_public=True)
        self.assert_request_id(fl, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/detail')
        for flavor in fl:
            self.assertIsInstance(flavor, self.flavor_type)

    def test_get_flavor_details(self):
        f = self.cs.flavors.get(1)
        self.assert_request_id(f, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/1')
        self.assertIsInstance(f, self.flavor_type)
        self.assertEqual(256, f.ram)
        self.assertEqual(10, f.disk)
        self.assertEqual(10, f.ephemeral)
        self.assertTrue(f.is_public)

    def test_get_flavor_details_alphanum_id(self):
        f = self.cs.flavors.get('aa1')
        self.assert_request_id(f, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/aa1')
        self.assertIsInstance(f, self.flavor_type)
        self.assertEqual(128, f.ram)
        self.assertEqual(0, f.disk)
        self.assertEqual(0, f.ephemeral)
        self.assertTrue(f.is_public)

    def test_get_flavor_details_diablo(self):
        f = self.cs.flavors.get(3)
        self.assert_request_id(f, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/3')
        self.assertIsInstance(f, self.flavor_type)
        self.assertEqual(256, f.ram)
        self.assertEqual(10, f.disk)
        self.assertEqual('N/A', f.ephemeral)
        self.assertEqual('N/A', f.is_public)

    def test_find(self):
        f = self.cs.flavors.find(ram=256)
        self.assert_request_id(f, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/flavors/detail')
        self.assertEqual('256 MiB Server', f.name)

        f = self.cs.flavors.find(disk=0)
        self.assert_request_id(f, fakes.FAKE_REQUEST_ID_LIST)
        self.assertEqual('128 MiB Server', f.name)

        self.assertRaises(exceptions.NotFound, self.cs.flavors.find,
                          disk=12345)

    @staticmethod
    def _create_body(name, ram, vcpus, disk, ephemeral, id, swap,
                     rxtx_factor, is_public):
        return {
            "flavor": {
                "name": name,
                "ram": ram,
                "vcpus": vcpus,
                "disk": disk,
                "OS-FLV-EXT-DATA:ephemeral": ephemeral,
                "id": id,
                "swap": swap,
                "rxtx_factor": rxtx_factor,
                "os-flavor-access:is_public": is_public,
            }
        }

    def test_create(self):
        f = self.cs.flavors.create("flavorcreate", 512, 1, 10, 1234,
                                   ephemeral=10, is_public=False)
        self.assert_request_id(f, fakes.FAKE_REQUEST_ID_LIST)

        body = self._create_body("flavorcreate", 512, 1, 10, 10, 1234, 0, 1.0,
                                 False)

        self.cs.assert_called('POST', '/flavors', body)
        self.assertIsInstance(f, self.flavor_type)

    def test_create_with_id_as_string(self):
        flavor_id = 'foobar'
        f = self.cs.flavors.create("flavorcreate", 512,
                                   1, 10, flavor_id, ephemeral=10,
                                   is_public=False)
        self.assert_request_id(f, fakes.FAKE_REQUEST_ID_LIST)

        body = self._create_body("flavorcreate", 512, 1, 10, 10, flavor_id, 0,
                                 1.0, False)

        self.cs.assert_called('POST', '/flavors', body)
        self.assertIsInstance(f, self.flavor_type)

    def test_create_ephemeral_ispublic_defaults(self):
        f = self.cs.flavors.create("flavorcreate", 512, 1, 10, 1234)
        self.assert_request_id(f, fakes.FAKE_REQUEST_ID_LIST)

        body = self._create_body("flavorcreate", 512, 1, 10, 0, 1234, 0,
                                 1.0, True)

        self.cs.assert_called('POST', '/flavors', body)
        self.assertIsInstance(f, self.flavor_type)

    def test_invalid_parameters_create(self):
        self.assertRaises(exceptions.CommandError, self.cs.flavors.create,
                          "flavorcreate", "invalid", 1, 10, 1234, swap=0,
                          ephemeral=0, rxtx_factor=1.0, is_public=True)
        self.assertRaises(exceptions.CommandError, self.cs.flavors.create,
                          "flavorcreate", 512, "invalid", 10, 1234, swap=0,
                          ephemeral=0, rxtx_factor=1.0, is_public=True)
        self.assertRaises(exceptions.CommandError, self.cs.flavors.create,
                          "flavorcreate", 512, 1, "invalid", 1234, swap=0,
                          ephemeral=0, rxtx_factor=1.0, is_public=True)
        self.assertRaises(exceptions.CommandError, self.cs.flavors.create,
                          "flavorcreate", 512, 1, 10, 1234, swap="invalid",
                          ephemeral=0, rxtx_factor=1.0, is_public=True)
        self.assertRaises(exceptions.CommandError, self.cs.flavors.create,
                          "flavorcreate", 512, 1, 10, 1234, swap=0,
                          ephemeral="invalid", rxtx_factor=1.0, is_public=True)
        self.assertRaises(exceptions.CommandError, self.cs.flavors.create,
                          "flavorcreate", 512, 1, 10, 1234, swap=0,
                          ephemeral=0, rxtx_factor="invalid", is_public=True)
        self.assertRaises(exceptions.CommandError, self.cs.flavors.create,
                          "flavorcreate", 512, 1, 10, 1234, swap=0,
                          ephemeral=0, rxtx_factor=1.0, is_public='invalid')

    def test_delete(self):
        ret = self.cs.flavors.delete("flavordelete")
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('DELETE', '/flavors/flavordelete')

    def test_delete_with_flavor_instance(self):
        f = self.cs.flavors.get(2)
        ret = self.cs.flavors.delete(f)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('DELETE', '/flavors/2')

    def test_delete_with_flavor_instance_method(self):
        f = self.cs.flavors.get(2)
        ret = f.delete()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('DELETE', '/flavors/2')

    def test_set_keys(self):
        f = self.cs.flavors.get(1)
        fk = f.set_keys({'k1': 'v1'})
        self.assert_request_id(fk, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('POST', '/flavors/1/os-extra_specs',
                              {"extra_specs": {'k1': 'v1'}})

    def test_set_with_valid_keys(self):
        valid_keys = ['key4', 'month.price', 'I-Am:AK-ey.44-',
                      'key with spaces and _']

        f = self.cs.flavors.get(4)
        for key in valid_keys:
            fk = f.set_keys({key: 'v4'})
            self.assert_request_id(fk, fakes.FAKE_REQUEST_ID_LIST)
            self.cs.assert_called('POST', '/flavors/4/os-extra_specs',
                                  {"extra_specs": {key: 'v4'}})

    def test_set_with_invalid_keys(self):
        invalid_keys = ['/1', '?1', '%1', '<', '>']

        f = self.cs.flavors.get(1)
        for key in invalid_keys:
            self.assertRaises(exceptions.CommandError, f.set_keys, {key: 'v1'})

    @mock.patch.object(flavors.FlavorManager, '_delete')
    def test_unset_keys(self, mock_delete):
        f = self.cs.flavors.get(1)
        keys = ['k1', 'k2']
        mock_delete.return_value = base.TupleWithMeta(
            (), fakes.FAKE_REQUEST_ID_LIST)
        fu = f.unset_keys(keys)
        self.assert_request_id(fu, fakes.FAKE_REQUEST_ID_LIST)
        mock_delete.assert_has_calls([
            mock.call("/flavors/1/os-extra_specs/k1"),
            mock.call("/flavors/1/os-extra_specs/k2")
        ])


class FlavorsTest_v2_55(utils.TestCase):
    """Tests creating/showing/updating a flavor with a description."""
    def setUp(self):
        super(FlavorsTest_v2_55, self).setUp()
        self.cs = fakes.FakeClient(api_versions.APIVersion('2.55'))

    def test_list_flavors(self):
        fl = self.cs.flavors.list()
        self.cs.assert_called('GET', '/flavors/detail')
        for flavor in fl:
            self.assertTrue(hasattr(flavor, 'description'),
                            "%s does not have a description set." % flavor)

    def test_list_flavors_undetailed(self):
        fl = self.cs.flavors.list(detailed=False)
        self.cs.assert_called('GET', '/flavors')
        for flavor in fl:
            self.assertTrue(hasattr(flavor, 'description'),
                            "%s does not have a description set." % flavor)

    def test_get_flavor_details(self):
        f = self.cs.flavors.get('with-description')
        self.cs.assert_called('GET', '/flavors/with-description')
        self.assertEqual('test description', f.description)

    def test_create(self):
        self.cs.flavors.create(
            'with-description', 512, 1, 10, 'with-description', ephemeral=10,
            is_public=False, description='test description')

        body = FlavorsTest._create_body(
            "with-description", 512, 1, 10, 10, 'with-description',
            0, 1.0, False)
        body['flavor']['description'] = 'test description'
        self.cs.assert_called('POST', '/flavors', body)

    def test_create_bad_version(self):
        """Tests trying to create a flavor with a description before 2.55."""
        self.cs.api_version = api_versions.APIVersion('2.54')
        self.assertRaises(exceptions.UnsupportedAttribute,
                          self.cs.flavors.create,
                          'with-description', 512, 1, 10, 'with-description',
                          description='test description')

    def test_update(self):
        updated_flavor = self.cs.flavors.update(
            'with-description', 'new description')
        body = {
            'flavor': {
                'description': 'new description'
            }
        }
        self.cs.assert_called('PUT', '/flavors/with-description', body)
        self.assertEqual('new description', updated_flavor.description)

    def test_update_bad_version(self):
        """Tests trying to update a flavor with a description before 2.55."""
        self.cs.api_version = api_versions.APIVersion('2.54')
        self.assertRaises(exceptions.VersionNotFoundForAPIMethod,
                          self.cs.flavors.update, 'foo', 'bar')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import time

import fixtures
from keystoneauth1 import identity
from keystoneauth1 import session as ksession
import openstack.config
import openstack.config.exceptions
import openstack.connection
from oslo_utils import uuidutils
import tempest.lib.cli.base
import testtools

import novaclient
import novaclient.api_versions
from novaclient import base
import novaclient.client
from novaclient.v2 import networks
import novaclient.v2.shell

BOOT_IS_COMPLETE = ("login as 'cirros' user. default password: "
                    "'gocubsgo'. use 'sudo' for root.")


# The following are simple filter functions that filter our available
# image / flavor list so that they can be used in standard testing.
def pick_flavor(flavors):
    """Given a flavor list pick a reasonable one."""
    for flavor_priority in ('m1.nano', 'm1.micro', 'm1.tiny', 'm1.small'):
        for flavor in flavors:
            if flavor.name == flavor_priority:
                return flavor
    raise NoFlavorException()


def pick_image(images):
    firstImage = None
    for image in images:
        firstImage = firstImage or image
        if image.name.startswith('cirros') and (
                image.name.endswith('-uec') or
                image.name.endswith('-disk.img')):
            return image

    # We didn't find the specific cirros image we'd like to use, so just use
    # the first available.
    if firstImage:
        return firstImage

    raise NoImageException()


def pick_network(networks):
    network_name = os.environ.get('OS_NOVACLIENT_NETWORK')
    if network_name:
        for network in networks:
            if network.name == network_name:
                return network
        raise NoNetworkException()
    return networks[0]


class NoImageException(Exception):
    """We couldn't find an acceptable image."""
    pass


class NoFlavorException(Exception):
    """We couldn't find an acceptable flavor."""
    pass


class NoNetworkException(Exception):
    """We couldn't find an acceptable network."""
    pass


class NoCloudConfigException(Exception):
    """We couldn't find a cloud configuration."""
    pass


CACHE = {}


class ClientTestBase(testtools.TestCase):
    """Base test class for read only python-novaclient commands.

    This is a first pass at a simple read only python-novaclient test. This
    only exercises client commands that are read only.

    This should test commands:
    * as a regular user
    * as a admin user
    * with and without optional parameters
    * initially just check return codes, and later test command outputs

    """
    COMPUTE_API_VERSION = None

    log_format = ('%(asctime)s %(process)d %(levelname)-8s '
                  '[%(name)s] %(message)s')

    def setUp(self):
        super(ClientTestBase, self).setUp()

        test_timeout = os.environ.get('OS_TEST_TIMEOUT', 0)
        try:
            test_timeout = int(test_timeout)
        except ValueError:
            test_timeout = 0
        if test_timeout > 0:
            self.useFixture(fixtures.Timeout(test_timeout, gentle=True))

        if (os.environ.get('OS_STDOUT_CAPTURE') == 'True' or
                os.environ.get('OS_STDOUT_CAPTURE') == '1'):
            stdout = self.useFixture(fixtures.StringStream('stdout')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stdout', stdout))
        if (os.environ.get('OS_STDERR_CAPTURE') == 'True' or
                os.environ.get('OS_STDERR_CAPTURE') == '1'):
            stderr = self.useFixture(fixtures.StringStream('stderr')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stderr', stderr))

        if (os.environ.get('OS_LOG_CAPTURE') != 'False' and
                os.environ.get('OS_LOG_CAPTURE') != '0'):
            self.useFixture(fixtures.LoggerFixture(nuke_handlers=False,
                                                   format=self.log_format,
                                                   level=None))

        # Collecting of credentials:
        #
        # Grab the cloud config from a user's clouds.yaml file.
        # First look for a functional_admin cloud, as this is a cloud
        # that the user may have defined for functional testing that has
        # admin credentials.
        # If that is not found, get the devstack config and override the
        # username and project_name to be admin so that admin credentials
        # will be used.
        #
        # Finally, fall back to looking for environment variables to support
        # existing users running these the old way. We should deprecate that
        # as tox 2.0 blanks out environment.
        #
        # TODO(sdague): while we collect this information in
        # tempest-lib, we do it in a way that's not available for top
        # level tests. Long term this probably needs to be in the base
        # class.
        openstack_config = openstack.config.OpenStackConfig()
        try:
            cloud_config = openstack_config.get_one_cloud('functional_admin')
        except openstack.config.exceptions.OpenStackConfigException:
            try:
                cloud_config = openstack_config.get_one_cloud(
                    'devstack', auth=dict(
                        username='admin', project_name='admin'))
            except openstack.config.exceptions.OpenStackConfigException:
                try:
                    cloud_config = openstack_config.get_one_cloud('envvars')
                except openstack.config.exceptions.OpenStackConfigException:
                    cloud_config = None

        if cloud_config is None:
            raise NoCloudConfigException(
                "Could not find a cloud named functional_admin or a cloud"
                " named devstack. Please check your clouds.yaml file and"
                " try again.")
        auth_info = cloud_config.config['auth']

        user = auth_info['username']
        passwd = auth_info['password']
        self.project_name = auth_info['project_name']
        auth_url = auth_info['auth_url']
        user_domain_id = auth_info['user_domain_id']
        self.project_domain_id = auth_info['project_domain_id']

        if 'insecure' in cloud_config.config:
            self.insecure = cloud_config.config['insecure']
        else:
            self.insecure = False
        self.cacert = cloud_config.config['cacert']
        self.cert = cloud_config.config['cert']

        auth = identity.Password(username=user,
                                 password=passwd,
                                 project_name=self.project_name,
                                 auth_url=auth_url,
                                 project_domain_id=self.project_domain_id,
                                 user_domain_id=user_domain_id)
        session = ksession.Session(
            cert=self.cert,
            auth=auth,
            verify=(self.cacert or not self.insecure)
        )

        self.client = self._get_novaclient(session)

        self.openstack = openstack.connection.Connection(session=session)

        # pick some reasonable flavor / image combo
        if "flavor" not in CACHE:
            CACHE["flavor"] = pick_flavor(self.client.flavors.list())
        if "image" not in CACHE:
            CACHE["image"] = pick_image(self.openstack.image.images())
        self.flavor = CACHE["flavor"]
        self.image = CACHE["image"]

        if "network" not in CACHE:
            # Get the networks from neutron.
            neutron_networks = self.openstack.network.networks()
            # Convert the neutron dicts to Network objects.
            nets = []
            for network in neutron_networks:
                nets.append(networks.Network(
                    networks.NeutronManager, network))
            # Keep track of whether or not there are multiple networks
            # available to the given tenant because if so, a specific
            # network ID has to be passed in on server create requests
            # otherwise the server POST will fail with a 409.
            CACHE['multiple_networks'] = len(nets) > 1
            CACHE["network"] = pick_network(nets)
        self.network = CACHE["network"]
        self.multiple_networks = CACHE['multiple_networks']

        # create a CLI client in case we'd like to do CLI
        # testing. tempest.lib does this really weird thing where it
        # builds a giant factory of all the CLIs that it knows
        # about. Eventually that should really be unwound into
        # something more sensible.
        cli_dir = os.environ.get(
            'OS_NOVACLIENT_EXEC_DIR',
            os.path.join(os.environ['TOX_ENV_DIR'], 'bin'))

        self.cli_clients = tempest.lib.cli.base.CLIClient(
            username=user,
            password=passwd,
            tenant_name=self.project_name,
            uri=auth_url,
            cli_dir=cli_dir,
            insecure=self.insecure)

    def _get_novaclient(self, session):
        nc = novaclient.client.Client("2", session=session)

        if self.COMPUTE_API_VERSION:
            if "min_api_version" not in CACHE:
                # Obtain supported versions by API side
                v = nc.versions.get_current()
                if not hasattr(v, 'version') or not v.version:
                    # API doesn't support microversions
                    CACHE["min_api_version"] = (
                        novaclient.api_versions.APIVersion("2.0"))
                    CACHE["max_api_version"] = (
                        novaclient.api_versions.APIVersion("2.0"))
                else:
                    CACHE["min_api_version"] = (
                        novaclient.api_versions.APIVersion(v.min_version))
                    CACHE["max_api_version"] = (
                        novaclient.api_versions.APIVersion(v.version))

            if self.COMPUTE_API_VERSION == "2.latest":
                requested_version = min(novaclient.API_MAX_VERSION,
                                        CACHE["max_api_version"])
            else:
                requested_version = novaclient.api_versions.APIVersion(
                    self.COMPUTE_API_VERSION)

            if not requested_version.matches(CACHE["min_api_version"],
                                             CACHE["max_api_version"]):
                msg = ("%s is not supported by Nova-API. Supported version" %
                       self.COMPUTE_API_VERSION)
                if CACHE["min_api_version"] == CACHE["max_api_version"]:
                    msg += ": %s" % CACHE["min_api_version"].get_string()
                else:
                    msg += "s: %s - %s" % (
                        CACHE["min_api_version"].get_string(),
                        CACHE["max_api_version"].get_string())
                self.skipTest(msg)

            nc.api_version = requested_version
        return nc

    def nova(self, action, flags='', params='', fail_ok=False,
             endpoint_type='publicURL', merge_stderr=False):
        if self.COMPUTE_API_VERSION:
            flags += " --os-compute-api-version %s " % self.COMPUTE_API_VERSION
        return self.cli_clients.nova(action, flags, params, fail_ok,
                                     endpoint_type, merge_stderr)

    def wait_for_volume_status(self, volume, status, timeout=60,
                               poll_interval=1):
        """Wait until volume reaches given status.

        :param volume: volume resource
        :param status: expected status of volume
        :param timeout: timeout in seconds
        :param poll_interval: poll interval in seconds
        """
        start_time = time.time()
        while time.time() - start_time < timeout:
            volume = self.openstack.block_storage.get_volume(volume)
            if volume.status == status:
                break
            time.sleep(poll_interval)
        else:
            self.fail("Volume %s did not reach status %s after %d s"
                      % (volume.id, status, timeout))

    def wait_for_server_os_boot(self, server_id, timeout=300,
                                poll_interval=1):
        """Wait until instance's operating system  is completely booted.

        :param server_id: uuid4 id of given instance
        :param timeout: timeout in seconds
        :param poll_interval: poll interval in seconds
        """
        start_time = time.time()
        console = None
        while time.time() - start_time < timeout:
            console = self.nova('console-log %s ' % server_id)
            if BOOT_IS_COMPLETE in console:
                break
            time.sleep(poll_interval)
        else:
            self.fail("Server %s did not boot after %d s.\nConsole:\n%s"
                      % (server_id, timeout, console))

    def wait_for_resource_delete(self, resource, manager,
                                 timeout=60, poll_interval=1):
        """Wait until getting the resource raises NotFound exception.

        :param resource: Resource object.
        :param manager: Manager object with get method.
        :param timeout: timeout in seconds
        :param poll_interval: poll interval in seconds
        """
        start_time = time.time()
        while time.time() - start_time < timeout:
            try:
                manager.get(resource)
            except Exception as e:
                if getattr(e, "http_status", None) == 404:
                    break
                else:
                    raise
            time.sleep(poll_interval)
        else:
            self.fail("The resource '%s' still exists." % base.getid(resource))

    def name_generate(self):
        """Generate randomized name for some entity."""
        # NOTE(andreykurilin): name_generator method is used for various
        #   resources (servers, flavors, volumes, keystone users, etc).
        #   Since the length of name has limits we cannot use the whole UUID,
        #   so the first 8 chars is taken from it.
        #   Based on the fact that the new name includes class and method
        #   names, 8 chars of uuid should be enough to prevent any conflicts,
        #   even if the single test will be launched in parallel thousand times
        return "%(prefix)s-%(test_cls)s-%(test_name)s" % {
            "prefix": uuidutils.generate_uuid()[:8],
            "test_cls": self.__class__.__name__,
            "test_name": self.id().rsplit(".", 1)[-1]
        }

    def _get_value_from_the_table(self, table, key):
        """Parses table to get desired value.

        EXAMPLE of the table:
        # +-------------+----------------------------------+
        # |   Property  |              Value               |
        # +-------------+----------------------------------+
        # | description |                                  |
        # |   enabled   |               True               |
        # |      id     | 582df899eabc47018c96713c2f7196ba |
        # |     name    |              admin               |
        # +-------------+----------------------------------+
        """
        lines = table.split("\n")
        for line in lines:
            if "|" in line:
                l_property, l_value = line.split("|")[1:3]
                if l_property.strip() == key:
                    return l_value.strip()
        raise ValueError("Property '%s' is missing from the table:\n%s" %
                         (key, table))

    def _get_column_value_from_single_row_table(self, table, column):
        """Get the value for the column in the single-row table

        Example table:

        +----------+-------------+----------+----------+
        | address  | cidr        | hostname | host     |
        +----------+-------------+----------+----------+
        | 10.0.0.3 | 10.0.0.0/24 | test     | myhost   |
        +----------+-------------+----------+----------+

        :param table: newline-separated table with |-separated cells
        :param column: name of the column to look for
        :raises: ValueError if the column value is not found
        """
        lines = table.split("\n")
        # Determine the column header index first.
        column_index = -1
        for line in lines:
            if "|" in line:
                if column_index == -1:
                    headers = line.split("|")[1:-1]
                    for index, header in enumerate(headers):
                        if header.strip() == column:
                            column_index = index
                            break
                else:
                    # We expect a single-row table so we should be able to get
                    # the value now using the column index.
                    return line.split("|")[1:-1][column_index].strip()

        raise ValueError("Unable to find value for column '%s'." % column)

    def _get_list_of_values_from_single_column_table(self, table, column):
        """Get the list of values for the column in the single-column table

        Example table:

        +------+
        | Tags |
        +------+
        | tag1 |
        | tag2 |
        +------+

        :param table: newline-separated table with |-separated cells
        :param column: name of the column to look for
        :raises: ValueError if the single column has some other name
        """
        lines = table.split("\n")
        column_name = None
        values = []
        for line in lines:
            if "|" in line:
                if not column_name:
                    column_name = line.split("|")[1].strip()
                    if column_name != column:
                        raise ValueError(
                            "The table has no column %(expected)s "
                            "but has column %(actual)s." % {
                                'expected': column, 'actual': column_name})
                else:
                    values.append(line.split("|")[1].strip())
        return values

    def _create_server(self, name=None, flavor=None, with_network=True,
                       add_cleanup=True, **kwargs):
        name = name or self.name_generate()
        if with_network:
            nics = [{"net-id": self.network.id}]
        else:
            nics = None
        flavor = flavor or self.flavor
        server = self.client.servers.create(name, self.image, flavor,
                                            nics=nics, **kwargs)
        if add_cleanup:
            self.addCleanup(server.delete)
        novaclient.v2.shell._poll_for_status(
            self.client.servers.get, server.id,
            'building', ['active'])
        return server

    def _wait_for_state_change(self, server_id, status):
        novaclient.v2.shell._poll_for_status(
            self.client.servers.get, server_id, None, [status],
            show_progress=False, poll_period=1, silent=True)

    def _get_project_id(self, name):
        """Obtain project id by project name."""
        return self.openstack.identity.find_project(
            name, ignore_missing=False
        ).id

    def _cleanup_server(self, server_id):
        """Deletes a server and waits for it to be gone."""
        self.client.servers.delete(server_id)
        self.wait_for_resource_delete(server_id, self.client.servers)

    def _get_absolute_limits(self):
        """Returns the absolute limits (quota usage) including reserved quota
        usage for the given tenant running the test.

        :return: A dict where the key is the limit (or usage) and value.
        """
        # The absolute limits are returned in a generator so convert to a dict.
        return {limit.name: limit.value
                for limit in self.client.limits.get(reserved=True).absolute}

    def _pick_alternate_flavor(self):
        """Given the flavor picked in the base class setup, this finds the
        opposite flavor to use for a resize test. For example, if m1.nano is
        the flavor, then use m1.micro, but those are only available if Tempest
        is configured. If m1.tiny, then use m1.small.
        """
        flavor_name = self.flavor.name
        if flavor_name == 'm1.nano':
            # This is an upsize test.
            return 'm1.micro'
        if flavor_name == 'm1.micro':
            # This is a downsize test.
            return 'm1.nano'
        if flavor_name == 'm1.tiny':
            # This is an upsize test.
            return 'm1.small'
        if flavor_name == 'm1.small':
            # This is a downsize test.
            return 'm1.tiny'
        self.fail('Unable to find alternate for flavor: %s' % flavor_name)


class ProjectTestBase(ClientTestBase):
    """Base test class for additional project and user creation which
    could be required in various test scenarios
    """

    def setUp(self):
        super(ProjectTestBase, self).setUp()
        user_name = uuidutils.generate_uuid()
        project_name = uuidutils.generate_uuid()
        password = 'password'

        project = self.openstack.identity.create_project(
            name=project_name,
            domain_id=self.project_domain_id)
        self.project_id = project.id
        self.addCleanup(
            self.openstack.identity.delete_project, self.project_id)

        self.user_id = self.openstack.identity.create_user(
            name=user_name, password=password,
            default_project=self.project_id).id

        for role in self.openstack.identity.roles():
            if "member" in role.name.lower():
                self.openstack.identity.assign_project_role_to_user(
                    project=self.project_id,
                    user=self.user_id,
                    role=role.id)
                break

        self.addCleanup(
            self.openstack.identity.delete_user, self.user_id)

        self.cli_clients_2 = tempest.lib.cli.base.CLIClient(
            username=user_name,
            password=password,
            tenant_name=project_name,
            uri=self.cli_clients.uri,
            cli_dir=self.cli_clients.cli_dir,
            insecure=self.insecure)

    def another_nova(self, action, flags='', params='', fail_ok=False,
                     endpoint_type='publicURL', merge_stderr=False):
        flags += " --os-compute-api-version %s " % self.COMPUTE_API_VERSION
        return self.cli_clients_2.nova(action, flags, params, fail_ok,
                                       endpoint_type, merge_stderr)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from novaclient.tests.unit.fixture_data import base


class FloatingFixture(base.Fixture):

    base_url = 'os-floating-ips'

    def setUp(self):
        super(FloatingFixture, self).setUp()

        floating_ips = [{'id': 1, 'fixed_ip': '10.0.0.1', 'ip': '11.0.0.1'},
                        {'id': 2, 'fixed_ip': '10.0.0.2', 'ip': '11.0.0.2'}]

        get_os_floating_ips = {'floating_ips': floating_ips}
        self.requests_mock.get(self.url(),
                               json=get_os_floating_ips,
                               headers=self.json_headers)

        for ip in floating_ips:
            get_os_floating_ip = {'floating_ip': ip}
            self.requests_mock.get(self.url(ip['id']),
                                   json=get_os_floating_ip,
                                   headers=self.json_headers)

            self.requests_mock.delete(self.url(ip['id']),
                                      headers=self.json_headers,
                                      status_code=204)

        def post_os_floating_ips(request, context):
            ip = floating_ips[0].copy()
            ip['pool'] = request.json().get('pool')
            return {'floating_ip': ip}
        self.requests_mock.post(self.url(),
                                json=post_os_floating_ips,
                                headers=self.json_headers)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient import api_versions
from novaclient.tests.unit.fixture_data import client
from novaclient.tests.unit.fixture_data import keypairs as data
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import keypairs


class KeypairsTest(utils.FixturedTestCase):

    client_fixture_class = client.V1
    data_fixture_class = data.V1

    def setUp(self):
        super(KeypairsTest, self).setUp()
        self.keypair_type = self._get_keypair_type()
        self.keypair_prefix = self._get_keypair_prefix()

    def _get_keypair_type(self):
        return keypairs.Keypair

    def _get_keypair_prefix(self):
        return keypairs.KeypairManager.keypair_prefix

    def test_get_keypair(self):
        kp = self.cs.keypairs.get('test')
        self.assert_request_id(kp, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/%s/test' % self.keypair_prefix)
        self.assertIsInstance(kp, keypairs.Keypair)
        self.assertEqual('test', kp.name)

    def test_list_keypairs(self):
        kps = self.cs.keypairs.list()
        self.assert_request_id(kps, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/%s' % self.keypair_prefix)
        for kp in kps:
            self.assertIsInstance(kp, keypairs.Keypair)

    def test_delete_keypair(self):
        kp = self.cs.keypairs.list()[0]
        ret = kp.delete()
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/%s/test' % self.keypair_prefix)
        ret = self.cs.keypairs.delete('test')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/%s/test' % self.keypair_prefix)
        ret = self.cs.keypairs.delete(kp)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('DELETE', '/%s/test' % self.keypair_prefix)


class KeypairsV2TestCase(KeypairsTest):
    def setUp(self):
        super(KeypairsV2TestCase, self).setUp()
        self.cs.api_version = api_versions.APIVersion("2.0")

    def test_create_keypair(self):
        name = "foo"
        kp = self.cs.keypairs.create(name)
        self.assert_request_id(kp, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/%s' % self.keypair_prefix,
                           body={'keypair': {'name': name}})
        self.assertIsInstance(kp, keypairs.Keypair)

    def test_import_keypair(self):
        name = "foo"
        pub_key = "fake-public-key"
        kp = self.cs.keypairs.create(name, pub_key)
        self.assert_request_id(kp, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/%s' % self.keypair_prefix,
                           body={'keypair': {'name': name,
                                             'public_key': pub_key}})
        self.assertIsInstance(kp, keypairs.Keypair)


class KeypairsV22TestCase(KeypairsTest):
    def setUp(self):
        super(KeypairsV22TestCase, self).setUp()
        self.cs.api_version = api_versions.APIVersion("2.2")

    def test_create_keypair(self):
        name = "foo"
        key_type = "some_type"
        kp = self.cs.keypairs.create(name, key_type=key_type)
        self.assert_request_id(kp, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/%s' % self.keypair_prefix,
                           body={'keypair': {'name': name,
                                             'type': key_type}})
        self.assertIsInstance(kp, keypairs.Keypair)

    def test_import_keypair(self):
        name = "foo"
        pub_key = "fake-public-key"
        kp = self.cs.keypairs.create(name, pub_key)
        self.assert_request_id(kp, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/%s' % self.keypair_prefix,
                           body={'keypair': {'name': name,
                                             'public_key': pub_key,
                                             'type': 'ssh'}})
        self.assertIsInstance(kp, keypairs.Keypair)


class KeypairsV35TestCase(KeypairsTest):
    def setUp(self):
        super(KeypairsV35TestCase, self).setUp()
        self.cs.api_version = api_versions.APIVersion("2.35")

    def test_list_keypairs(self):
        kps = self.cs.keypairs.list(user_id='test_user', marker='test_kp',
                                    limit=3)
        self.assert_request_id(kps, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET',
                           '/%s?limit=3&marker=test_kp&user_id=test_user'
                           % self.keypair_prefix)
        for kp in kps:
            self.assertIsInstance(kp, keypairs.Keypair)


class KeypairsV92TestCase(KeypairsTest):
    def setUp(self):
        super(KeypairsV92TestCase, self).setUp()
        self.cs.api_version = api_versions.APIVersion("2.92")

    def test_create_keypair(self):
        name = "foo"
        key_type = "some_type"
        public_key = "fake-public-key"
        kp = self.cs.keypairs.create(name, public_key=public_key,
                                     key_type=key_type)
        self.assert_request_id(kp, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('POST', '/%s' % self.keypair_prefix,
                           body={'keypair': {'name': name,
                                             'public_key': public_key,
                                             'type': key_type}})
        self.assertIsInstance(kp, keypairs.Keypair)

    def test_create_keypair_without_pubkey(self):
        name = "foo"
        key_type = "some_type"
        self.assertRaises(TypeError,
                          self.cs.keypairs.create, name, key_type=key_type)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from novaclient.tests.unit.fixture_data import base


class Fixture(base.Fixture):

    base_url = 'os-aggregates'

    def setUp(self):
        super(Fixture, self).setUp()

        get_os_aggregates = {"aggregates": [
            {'id': '1',
             'name': 'test',
             'availability_zone': 'nova1'},
            {'id': '2',
             'name': 'test2',
             'availability_zone': 'nova1'},
        ]}

        self.requests_mock.get(self.url(),
                               json=get_os_aggregates,
                               headers=self.json_headers)

        get_aggregates_1 = {'aggregate': get_os_aggregates['aggregates'][0]}

        self.requests_mock.post(self.url(),
                                json=get_aggregates_1,
                                headers=self.json_headers)

        for agg_id in (1, 2):
            for method in ('GET', 'PUT'):
                self.requests_mock.register_uri(method, self.url(agg_id),
                                                json=get_aggregates_1,
                                                headers=self.json_headers)

            self.requests_mock.post(self.url(agg_id, 'action'),
                                    json=get_aggregates_1,
                                    headers=self.json_headers)

        self.requests_mock.delete(self.url(1), status_code=202,
                                  headers=self.json_headers)

        self.requests_mock.register_uri('POST', self.url(1),
                                        json={},
                                        headers=self.json_headers)
        self.requests_mock.post(self.url(1, 'images'),
                                json={},
                                headers=self.json_headers)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from novaclient.tests.unit.fixture_data import base


class V1(base.Fixture):

    base_url = 'os-availability-zone'

    zone_info_key = 'availabilityZoneInfo'
    zone_name_key = 'zoneName'
    zone_state_key = 'zoneState'

    def setUp(self):
        super(V1, self).setUp()

        get_os_availability_zone = {
            self.zone_info_key: [
                {
                    self.zone_name_key: "zone-1",
                    self.zone_state_key: {"available": True},
                    "hosts": None
                },
                {
                    self.zone_name_key: "zone-2",
                    self.zone_state_key: {"available": False},
                    "hosts": None
                }
            ]
        }

        self.requests_mock.get(self.url(),
                               json=get_os_availability_zone,
                               headers=self.json_headers)

        get_os_zone_detail = {
            self.zone_info_key: [
                {
                    self.zone_name_key: "zone-1",
                    self.zone_state_key: {"available": True},
                    "hosts": {
                        "fake_host-1": {
                            "nova-compute": {
                                "active": True,
                                "available": True,
                                "updated_at": '2012-12-26 14:45:25'
                            }
                        }
                    }
                },
                {
                    self.zone_name_key: "internal",
                    self.zone_state_key: {"available": True},
                    "hosts": {
                        "fake_host-1": {
                            "nova-sched": {
                                "active": True,
                                "available": True,
                                "updated_at": '2012-12-26 14:45:25'
                            }
                        },
                        "fake_host-2": {
                            "nova-network": {
                                "active": True,
                                "available": False,
                                "updated_at": '2012-12-26 14:45:24'
                            }
                        }
                    }
                },
                {
                    self.zone_name_key: "zone-2",
                    self.zone_state_key: {"available": False},
                    "hosts": None
                }
            ]
        }

        self.requests_mock.get(self.url('detail'),
                               json=get_os_zone_detail,
                               headers=self.json_headers)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from novaclient import api_versions
from novaclient.tests.unit import fakes
from novaclient.tests.unit.fixture_data import base


class V1(base.Fixture):

    api_version = '2.1'
    base_url = 'os-keypairs'

    def setUp(self):
        super(V1, self).setUp()
        api_version = api_versions.APIVersion(self.api_version)

        keypair = {'fingerprint': 'FAKE_KEYPAIR', 'name': 'test'}

        headers = self.json_headers

        self.requests_mock.get(self.url(),
                               json={'keypairs': [keypair]},
                               headers=headers)

        self.requests_mock.get(self.url('test'),
                               json={'keypair': keypair},
                               headers=headers)

        self.requests_mock.delete(self.url('test'),
                                  status_code=202,
                                  headers=headers)

        def post_os_keypairs(request, context):
            body = request.json()
            assert list(body) == ['keypair']
            if api_version >= api_versions.APIVersion("2.92"):
                # In 2.92, public_key becomes mandatory
                required = ['name', 'public_key']
            else:
                required = ['name']
            fakes.assert_has_keys(body['keypair'],
                                  required=required)
            return {'keypair': keypair}

        self.requests_mock.post(self.url(),
                                json=post_os_keypairs,
                                headers=headers)



# Copyright 2013 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient import api_versions
from novaclient import exceptions
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes


class InstanceUsageAuditLogTests(utils.TestCase):
    def setUp(self):
        super(InstanceUsageAuditLogTests, self).setUp()
        self.cs = fakes.FakeClient(api_versions.APIVersion("2.1"))

    def test_instance_usage_audit_log(self):
        audit_log = self.cs.instance_usage_audit_log.get()
        self.assert_request_id(audit_log, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/os-instance_usage_audit_log')

    def test_instance_usage_audit_log_with_before(self):
        audit_log = self.cs.instance_usage_audit_log.get(
            before='2016-12-10 13:59:59.999999')
        self.assert_request_id(audit_log, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called(
            'GET',
            '/os-instance_usage_audit_log/2016-12-10%2013%3A59%3A59.999999')

    def test_instance_usage_audit_log_with_before_unicode(self):
        before = '\\u5de5\\u4f5c'
        self.assertRaises(exceptions.BadRequest,
                          self.cs.instance_usage_audit_log.get, before)



# Copyright 2015 IBM Corp.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import exceptions

from novaclient.tests.functional import base


class TestConsolesNovaClient(base.ClientTestBase):
    """Consoles functional tests."""

    COMPUTE_API_VERSION = "2.1"

    def _test_console_get(self, command, expected_response_type):
        server = self._create_server()
        completed_command = command % server.id

        try:
            output = self.nova(completed_command)
            # if we didn't fail, check that the expected response type is in
            # the output
            console_type = self._get_column_value_from_single_row_table(
                output, 'Type')
            self.assertEqual(expected_response_type, console_type, output)
        except exceptions.CommandFailed as cf:
            self.assertIn('HTTP 400', str(cf.stderr))

    def _test_vnc_console_get(self):
        self._test_console_get('get-vnc-console %s novnc', 'novnc')

    def _test_spice_console_get(self):
        self._test_console_get('get-spice-console %s spice-html5',
                               'spice-html5')

    def _test_rdp_console_get(self):
        self._test_console_get('get-rdp-console %s rdp-html5', 'rdp-html5')

    def _test_serial_console_get(self):
        self._test_console_get('get-serial-console %s', 'serial')

    def test_vnc_console_get(self):
        self._test_vnc_console_get()

    def test_spice_console_get(self):
        self._test_spice_console_get()

    def test_rdp_console_get(self):
        self._test_rdp_console_get()

    def test_serial_console_get(self):
        self._test_serial_console_get()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from oslo_utils import timeutils

from novaclient.tests.functional import base


class TestServersBootNovaClient(base.ClientTestBase):
    """Servers boot functional tests."""

    COMPUTE_API_VERSION = "2.1"

    def _boot_server_with_legacy_bdm(self, bdm_params=()):
        volume_size = 1
        volume_name = self.name_generate()
        volume = self.openstack.block_storage.create_volume(
            size=volume_size, name=volume_name, image_id=self.image.id)
        self.wait_for_volume_status(volume, "available")

        if (len(bdm_params) >= 3 and bdm_params[2] == '1'):
            delete_volume = False
        else:
            delete_volume = True

        bdm_params = ':'.join(bdm_params)
        if bdm_params:
            bdm_params = ''.join((':', bdm_params))

        params = (
            "%(name)s --flavor %(flavor)s --poll "
            "--block-device-mapping vda=%(volume_id)s%(bdm_params)s" % {
                "name": self.name_generate(), "flavor":
                    self.flavor.id,
                "volume_id": volume.id,
                "bdm_params": bdm_params})
        # check to see if we have to pass in a network id
        if self.multiple_networks:
            params += ' --nic net-id=%s' % self.network.id
        server_info = self.nova("boot", params=params)
        server_id = self._get_value_from_the_table(server_info, "id")

        self.client.servers.delete(server_id)
        self.wait_for_resource_delete(server_id, self.client.servers)

        if delete_volume:
            self.openstack.block_storage.delete_volume(volume)
            self.openstack.block_storage.wait_for_delete(volume)

    def test_boot_server_with_legacy_bdm(self):
        # bdm v1 format
        # <id>:<type>:<size(GiB)>:<delete-on-terminate>
        # params = (type, size, delete-on-terminate)
        params = ('', '', '1')
        self._boot_server_with_legacy_bdm(bdm_params=params)

    def test_boot_server_with_legacy_bdm_volume_id_only(self):
        self._boot_server_with_legacy_bdm()

    def test_boot_server_with_net_name(self):
        server_info = self.nova("boot", params=(
            "%(name)s --flavor %(flavor)s --image %(image)s --poll "
            "--nic net-name=%(net-name)s" % {"name": self.name_generate(),
                                             "image": self.image.id,
                                             "flavor": self.flavor.id,
                                             "net-name": self.network.name}))
        server_id = self._get_value_from_the_table(server_info, "id")

        self.client.servers.delete(server_id)
        self.wait_for_resource_delete(server_id, self.client.servers)

    def test_boot_server_using_image_with(self):
        """Scenario test which does the following:

        1. Create a server.
        2. Create a snapshot image of the server with a special meta key.
        3. Create a second server using the --image-with option using the meta
           key stored in the snapshot image created in step 2.
        """
        params = (
            '--flavor %(flavor)s --image %(image)s --poll '
            'image-with-server-1' % {'image': self.image.id,
                                     'flavor': self.flavor.id})
        # check to see if we have to pass in a network id
        if self.multiple_networks:
            params += ' --nic net-id=%s' % self.network.id
        # create the first server and wait for it to be active
        server_info = self.nova('boot', params=params)
        server_id = self._get_value_from_the_table(server_info, 'id')
        self.addCleanup(self._cleanup_server, server_id)

        # create a snapshot of the server with an image metadata key
        snapshot_info = self.nova('image-create', params=(
            '--metadata image_with_meta=%(meta_value)s '
            '--show --poll %(server_id)s image-with-snapshot' % {
                'meta_value': server_id,
                'server_id': server_id}))

        # get the snapshot image id out of the output table for the second
        # server create request
        snapshot_id = self._get_value_from_the_table(snapshot_info, 'id')
        self.addCleanup(self.openstack.image.delete_image, snapshot_id)

        # verify the metadata was set on the snapshot image
        meta_value = self._get_value_from_the_table(
            snapshot_info, 'image_with_meta')
        self.assertEqual(server_id, meta_value)

        params = (
            '--flavor %(flavor)s --image-with image_with_meta=%(meta_value)s '
            '--poll image-with-server-2' % {'meta_value': server_id,
                                            'flavor': self.flavor.id})
        # check to see if we have to pass in a network id
        if self.multiple_networks:
            params += ' --nic net-id=%s' % self.network.id
        # create the second server using --image-with
        server_info = self.nova('boot', params=params)
        server_id = self._get_value_from_the_table(server_info, 'id')
        self.addCleanup(self._cleanup_server, server_id)


class TestServersListNovaClient(base.ClientTestBase):
    """Servers list functional tests."""

    COMPUTE_API_VERSION = "2.1"

    def _create_servers(self, name, number):
        return [self._create_server(name) for i in range(number)]

    def test_list_with_limit(self):
        name = self.name_generate()
        self._create_servers(name, 2)
        output = self.nova("list", params="--limit 1 --name %s" % name)
        # Cut header and footer of the table
        servers = output.split("\n")[3:-2]
        self.assertEqual(1, len(servers), output)

    def test_list_with_changes_since(self):
        now = datetime.datetime.isoformat(timeutils.utcnow())
        name = self.name_generate()
        self._create_servers(name, 1)
        output = self.nova("list", params="--changes-since %s" % now)
        self.assertIn(name, output, output)
        now = datetime.datetime.isoformat(timeutils.utcnow())
        output = self.nova("list", params="--changes-since %s" % now)
        self.assertNotIn(name, output, output)

    def test_list_all_servers(self):
        name = self.name_generate()
        precreated_servers = self._create_servers(name, 3)
        # there are no possibility to exceed the limit on API side, so just
        # check that "-1" limit processes by novaclient side
        output = self.nova("list", params="--limit -1 --name %s" % name)
        # Cut header and footer of the table
        for server in precreated_servers:
            self.assertIn(server.id, output)

    def test_list_minimal(self):
        server = self._create_server()
        server_output = self.nova("list --minimal")
        # The only fields output are "ID" and "Name"
        output_uuid = self._get_column_value_from_single_row_table(
            server_output, 'ID')
        output_name = self._get_column_value_from_single_row_table(
            server_output, 'Name')
        self.assertEqual(output_uuid, server.id)
        self.assertEqual(output_name, server.name)



# Copyright (C) 2016, Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import exceptions

from novaclient.tests.functional import base


class TestBlockDeviceTaggingCLIError(base.ClientTestBase):
    """Negative test that asserts that creating a server with a tagged
    block device with a specific microversion will fail.
    """

    COMPUTE_API_VERSION = "2.31"

    def test_boot_server_with_tagged_block_devices_with_error(self):
        try:
            output = self.nova('boot', params=(
                '%(name)s --flavor %(flavor)s --poll '
                '--nic net-id=%(net-uuid)s '
                '--block-device '
                'source=image,dest=volume,id=%(image)s,size=1,bootindex=0,'
                'shutdown=remove,tag=bar' % {'name': self.name_generate(),
                                             'flavor': self.flavor.id,
                                             'net-uuid': self.network.id,
                                             'image': self.image.id}))
        except exceptions.CommandFailed as e:
            self.assertIn("ERROR (CommandError): "
                          "'tag' in block device mapping is not supported "
                          "in API version %s." % self.COMPUTE_API_VERSION,
                          str(e))
        else:
            server_id = self._get_value_from_the_table(output, 'id')
            self.client.servers.delete(server_id)
            self.wait_for_resource_delete(server_id, self.client.servers)
            self.fail("Booting a server with block device tag is not failed.")


class TestNICDeviceTaggingCLIError(base.ClientTestBase):
    """Negative test that asserts that creating a server with a tagged
    nic with a specific microversion will fail.
    """

    COMPUTE_API_VERSION = "2.31"

    def test_boot_server_with_tagged_nic_devices_with_error(self):
        try:
            output = self.nova('boot', params=(
                '%(name)s --flavor %(flavor)s --poll '
                '--nic net-id=%(net-uuid)s,tag=foo '
                '--block-device '
                'source=image,dest=volume,id=%(image)s,size=1,bootindex=0,'
                'shutdown=remove' % {'name': self.name_generate(),
                                     'flavor': self.flavor.id,
                                     'net-uuid': self.network.id,
                                     'image': self.image.id}))
        except exceptions.CommandFailed as e:
            self.assertIn('Invalid nic argument', str(e))
        else:
            server_id = self._get_value_from_the_table(output, 'id')
            self.client.servers.delete(server_id)
            self.wait_for_resource_delete(server_id, self.client.servers)
            self.fail("Booting a server with network interface tag "
                      "is not failed.")


class TestBlockDeviceTaggingCLI(base.ClientTestBase):
    """Tests that creating a server with a tagged block device will work
    with the 2.32 microversion, where the feature was originally added.
    """

    COMPUTE_API_VERSION = "2.32"

    def test_boot_server_with_tagged_block_devices(self):
        server_info = self.nova('boot', params=(
            '%(name)s --flavor %(flavor)s --poll '
            '--nic net-id=%(net-uuid)s '
            '--block-device '
            'source=image,dest=volume,id=%(image)s,size=1,bootindex=0,'
            'shutdown=remove,tag=bar' % {'name': self.name_generate(),
                                         'flavor': self.flavor.id,
                                         'net-uuid': self.network.id,
                                         'image': self.image.id}))
        server_id = self._get_value_from_the_table(server_info, 'id')
        self.client.servers.delete(server_id)
        self.wait_for_resource_delete(server_id, self.client.servers)


class TestNICDeviceTaggingCLI(base.ClientTestBase):
    """Tests that creating a server with a tagged nic will work
    with the 2.32 microversion, where the feature was originally added.
    """

    COMPUTE_API_VERSION = "2.32"

    def test_boot_server_with_tagged_nic_devices(self):
        server_info = self.nova('boot', params=(
            '%(name)s --flavor %(flavor)s --poll '
            '--nic net-id=%(net-uuid)s,tag=foo '
            '--block-device '
            'source=image,dest=volume,id=%(image)s,size=1,bootindex=0,'
            'shutdown=remove' % {'name': self.name_generate(),
                                 'flavor': self.flavor.id,
                                 'net-uuid': self.network.id,
                                 'image': self.image.id}))
        server_id = self._get_value_from_the_table(server_info, 'id')
        self.client.servers.delete(server_id)
        self.wait_for_resource_delete(server_id, self.client.servers)


class TestDeviceTaggingCLIV233(TestBlockDeviceTaggingCLIError,
                               TestNICDeviceTaggingCLI):
    """Tests that in microversion 2.33, creating a server with a tagged
    block device will fail, but creating a server with a tagged nic will
    succeed.
    """

    COMPUTE_API_VERSION = "2.33"


class TestDeviceTaggingCLIV236(TestBlockDeviceTaggingCLIError,
                               TestNICDeviceTaggingCLI):
    """Tests that in microversion 2.36, creating a server with a tagged
    block device will fail, but creating a server with a tagged nic will
    succeed. This is testing the boundary before 2.37 where nic tagging
    was broken.
    """

    COMPUTE_API_VERSION = "2.36"


class TestDeviceTaggingCLIV237(TestBlockDeviceTaggingCLIError,
                               TestNICDeviceTaggingCLIError):
    """Tests that in microversion 2.37, creating a server with either a
    tagged block device or tagged nic would fail.
    """

    COMPUTE_API_VERSION = "2.37"


class TestDeviceTaggingCLIV241(TestBlockDeviceTaggingCLIError,
                               TestNICDeviceTaggingCLIError):
    """Tests that in microversion 2.41, creating a server with either a
    tagged block device or tagged nic would fail. This is testing the
    boundary before 2.42 where block device tags and nic tags were fixed
    for server create requests.
    """

    COMPUTE_API_VERSION = "2.41"


class TestDeviceTaggingCLIV242(TestBlockDeviceTaggingCLI,
                               TestNICDeviceTaggingCLI):
    """Tests that in microversion 2.42 you could once again create a server
    with a tagged block device or a tagged nic.
    """

    COMPUTE_API_VERSION = "2.42"



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional import base


class TestInstanceCLI(base.ClientTestBase):

    COMPUTE_API_VERSION = "2.1"

    def test_attach_volume(self):
        """Test we can attach a volume via the cli.

        This test was added after bug 1423695. That bug exposed
        inconsistencies in how to talk to API services from the CLI
        vs. API level. The volumes api calls that were designed to
        populate the completion cache were incorrectly routed to the
        Nova endpoint. Novaclient volumes support actually talks to
        Cinder endpoint directly.

        This would case volume-attach to return a bad error code,
        however it does this *after* the attach command is correctly
        dispatched. So the volume-attach still works, but the user is
        presented a 404 error.

        This test ensures we can do a through path test of: boot,
        create volume, attach volume, detach volume, delete volume,
        destroy.

        """
        name = self.name_generate()

        # Boot via the cli, as we're primarily testing the cli in this test
        self.nova('boot',
                  params="--flavor %s --image %s %s --nic net-id=%s --poll" %
                  (self.flavor.name, self.image.name, name, self.network.id))

        # Be nice about cleaning up, however, use the API for this to avoid
        # parsing text.
        servers = self.client.servers.list(search_opts={"name": name})
        # the name is a random uuid, there better only be one
        self.assertEqual(1, len(servers), servers)
        server = servers[0]
        self.addCleanup(server.delete)

        # create a volume for attachment
        volume = self.openstack.block_storage.create_volume(size=1)
        self.addCleanup(self.openstack.block_storage.delete_volume, volume)

        # allow volume to become available
        self.wait_for_volume_status(volume, 'available')

        # attach the volume
        self.nova('volume-attach', params="%s %s" % (name, volume.id))

        # volume needs to transition to 'in-use' to be attached
        self.wait_for_volume_status(volume, 'in-use')

        # clean up on success
        self.nova('volume-detach', params="%s %s" % (name, volume.id))
        self.wait_for_volume_status(volume, 'available')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from novaclient.tests.unit.fixture_data import base


class Fixture(base.Fixture):

    base_url = 'os-server-groups'

    def setUp(self):
        super(Fixture, self).setUp()

        server_groups = [
            {
                "members": [],
                "metadata": {},
                "id": "2cbd51f4-fafe-4cdb-801b-cf913a6f288b",
                "policies": [],
                "name": "ig1"
            },
            {
                "members": [],
                "metadata": {},
                "id": "4473bb03-4370-4bfb-80d3-dc8cffc47d94",
                "policies": ["anti-affinity"],
                "name": "ig2"
            },
            {
                "members": [],
                "metadata": {"key": "value"},
                "id": "31ab9bdb-55e1-4ac3-b094-97eeb1b65cc4",
                "policies": [], "name": "ig3"
            },
            {
                "members": ["2dccb4a1-02b9-482a-aa23-5799490d6f5d"],
                "metadata": {},
                "id": "4890bb03-7070-45fb-8453-d34556c87d94",
                "policies": ["anti-affinity"],
                "name": "ig2"
            }
        ]

        other_project_server_groups = [
            {
                "members": [],
                "metadata": {},
                "id": "11111111-1111-1111-1111-111111111111",
                "policies": [],
                "name": "ig4"
            },
            {
                "members": [],
                "metadata": {},
                "id": "22222222-2222-2222-2222-222222222222",
                "policies": ["anti-affinity"],
                "name": "ig5"
            },
            {
                "members": [],
                "metadata": {"key": "value"},
                "id": "33333333-3333-3333-3333-333333333333",
                "policies": [], "name": "ig6"
            },
            {
                "members": ["2dccb4a1-02b9-482a-aa23-5799490d6f5d"],
                "metadata": {},
                "id": "44444444-4444-4444-4444-444444444444",
                "policies": ["anti-affinity"],
                "name": "ig5"
            }
        ]

        headers = self.json_headers

        self.requests_mock.get(self.url(),
                               json={'server_groups': server_groups},
                               headers=headers)

        self.requests_mock.get(self.url(all_projects=True),
                               json={'server_groups': server_groups +
                                     other_project_server_groups},
                               headers=headers)

        self.requests_mock.get(self.url(limit=2, offset=1),
                               json={'server_groups': server_groups[1:3]},
                               headers=headers)

        server = server_groups[0]

        def _register(method, *args):
            self.requests_mock.register_uri(method,
                                            self.url(*args),
                                            json={'server_group': server},
                                            headers=headers)

        _register('POST')
        _register('POST', server['id'])
        _register('GET', server['id'])
        _register('PUT', server['id'])
        _register('POST', server['id'], '/action')

        self.requests_mock.delete(self.url(server['id']),
                                  status_code=202,
                                  headers=headers)



# Copyright 2015 IBM Corp.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional.v2.legacy import test_consoles


class TestConsolesNovaClientV26(test_consoles.TestConsolesNovaClient):
    """Consoles functional tests for >=v2.6 api microversions."""

    COMPUTE_API_VERSION = "2.6"

    def test_vnc_console_get(self):
        self._test_vnc_console_get()

    def test_spice_console_get(self):
        self._test_spice_console_get()

    def test_rdp_console_get(self):
        self._test_rdp_console_get()

    def test_serial_console_get(self):
        self._test_serial_console_get()


class TestConsolesNovaClientV28(test_consoles.TestConsolesNovaClient):
    """Consoles functional tests for >=v2.8 api microversions."""

    COMPUTE_API_VERSION = "2.8"

    def test_webmks_console_get(self):
        self._test_console_get('get-mks-console %s ', 'webmks')
